@@ -17,14 +17,14 @@
 #include "KNotificationBackend.h"
 
 #include "core/support/Amarok.h"
-#include "SvgHandler.h"
-#include "core/support/Debug.h"
-#include "EngineController.h"
 #include "core/meta/Meta.h"
+#include "SvgHandler.h"
+#include "EngineController.h"
+
+#include <KLocale>
 
 #include <QTimer>
 
-
 namespace Amarok
 {
     KNotificationBackend*
@@ -44,21 +44,22 @@ namespace Amarok
 }
 
 Amarok::KNotificationBackend::KNotificationBackend()
-    : EngineObserver( The::engineController() )
-    , m_notify( 0 )
+    : m_notify( 0 )
     , m_enabled( false )
 {
-    DEBUG_BLOCK
-
     m_timer = new QTimer( this );
     m_timer->setSingleShot( true );
     connect( m_timer, SIGNAL( timeout() ), this, SLOT( showCurrentTrack() ) );
+
+    EngineController *engine = The::engineController();
+
+    connect( engine, SIGNAL( trackPlaying( Meta::TrackPtr ) ),
+             this, SLOT( trackPlaying() ) );
+
 }
 
 Amarok::KNotificationBackend::~KNotificationBackend()
 {
-    DEBUG_BLOCK
-
     if (m_notify)
         m_notify->close();
 }
@@ -83,44 +84,24 @@ Amarok::KNotificationBackend::notificationClosed()
 }
 
 void
-Amarok::KNotificationBackend::engineStateChanged( Phonon::State state, Phonon::State /*oldState*/ )
+Amarok::KNotificationBackend::trackPlaying()
 {
-    switch( state )
-    {
-        case Phonon::PlayingState:
-        case Phonon::StoppedState:
-        case Phonon::PausedState:
-        case Phonon::LoadingState:
-            break;
-        case Phonon::ErrorState:
-            if( m_timer->isActive() )
-                m_timer->stop(); // Do not notify if track cannot be played
-        case Phonon::BufferingState:
-            break;
-    }
-}
-
-void
-Amarok::KNotificationBackend::engineNewTrackPlaying()
-{
-    DEBUG_BLOCK
-
     if( !m_enabled )
         return; // KNotify is disabled, so don't start timer
 
-    m_timer->start( 3000 ); // Wait some time to display the correct cover
+    m_timer->start( 3000 ); // Wait some time to display the correct cover and also to check if phonon really managed to play the track
 }
 
 void
 Amarok::KNotificationBackend::showCurrentTrack() // slot
 {
-    DEBUG_BLOCK
-
     if( !m_enabled )
         return;
 
-    Meta::TrackPtr track = The::engineController()->currentTrack();
-    if( track )
+    EngineController *engine = The::engineController();
+
+    Meta::TrackPtr track = engine->currentTrack();
+    if( engine->isPlaying() && track )
     {
         if( m_notify ) {
             m_notify->close(); // Close old notification (when switching quickly between tracks)
@@ -129,12 +110,13 @@ Amarok::KNotificationBackend::showCurrentTrack() // slot
         m_notify = new KNotification( "trackChange" );
         connect( m_notify, SIGNAL(closed()), this, SLOT(notificationClosed()) );
 
-        if( track->album() )
-            m_notify->setPixmap( The::svgHandler()->imageWithBorder( track->album(), 80 ) );
+        Meta::AlbumPtr album = track->album();
+        if( album )
+            m_notify->setPixmap( The::svgHandler()->imageWithBorder( album, 80 ) );
 
         m_notify->setTitle( i18n( "Now playing" ) );
 
-        m_notify->setText( The::engineController()->prettyNowPlaying() );
+        m_notify->setText( engine->prettyNowPlaying() );
         m_notify->sendEvent();
     }
 }
