@@ -19,7 +19,7 @@
 #include "BrowserBreadcrumbItem.h"
 #include "BrowserCategoryList.h"
 
-#include "Debug.h"
+#include "core/support/Debug.h"
 
 #include "ToolBar.h"
 
