@@ -50,7 +50,7 @@ SingleCollectionTreeItemModel::setLevels( const QList<int> &levelType )
 
     updateHeaderText();
     m_expandedItems.clear();
-    m_expandedVariousArtistsNodes.clear();
+    m_expandedSpecialNodes.clear();
     reset(); //resets the whole model, as the data changed
 }
 
