@@ -35,7 +35,7 @@
 #include <KButtonGroup>
 #include <KIcon>
 #include <KLineEdit>
-
+#include <KConfigGroup>
 #include <QButtonGroup>
 #include <QCheckBox>
 #include <QGridLayout>
@@ -67,17 +67,6 @@ PlaylistBrowserCategory::PlaylistBrowserCategory( int playlistCategory,
     m_toolBar = new KToolBar( this, false, false );
     m_toolBar->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
 
-    m_byProviderProxy = new PlaylistsByProviderProxy( model, PlaylistBrowserModel::ProviderColumn );
-    m_byFolderProxy = new PlaylistsInFoldersProxy( model );
-
-    m_filterProxy = new QSortFilterProxyModel( this );
-    m_filterProxy->setDynamicSortFilter( true );
-    m_filterProxy->setFilterKeyColumn( PlaylistBrowserModel::ProviderColumn );
-
-    m_playlistView = new PlaylistBrowserView( m_filterProxy, this );
-    m_defaultItemDelegate = m_playlistView->itemDelegate();
-    m_byProviderDelegate = new PlaylistTreeItemDelegate( m_playlistView );
-
     //a QWidget with minimumExpanding makes the next button right aligned.
     QWidget *spacerWidget = new QWidget( this );
     spacerWidget->setSizePolicy( QSizePolicy::MinimumExpanding,
@@ -107,7 +96,20 @@ PlaylistBrowserCategory::PlaylistBrowserCategory( int playlistCategory,
 
     m_toolBar->addSeparator();
 
-    toggleView( toggleAction->isChecked() );
+    m_byProviderProxy = new PlaylistsByProviderProxy( model, PlaylistBrowserModel::ProviderColumn );
+    m_byFolderProxy = new PlaylistsInFoldersProxy( model );
+
+    QtGroupingProxy *filterSourceModel = toggleAction->isChecked()
+        ? static_cast<QtGroupingProxy*>(m_byFolderProxy)
+        : static_cast<QtGroupingProxy*>(m_byProviderProxy);
+    m_filterProxy = new QSortFilterProxyModel( this );
+    m_filterProxy->setSourceModel( filterSourceModel );
+    m_filterProxy->setDynamicSortFilter( true );
+    m_filterProxy->setFilterKeyColumn( PlaylistBrowserModel::ProviderColumn );
+
+    m_playlistView = new PlaylistBrowserView( m_filterProxy, this );
+    m_defaultItemDelegate = m_playlistView->itemDelegate();
+    m_byProviderDelegate = new PlaylistTreeItemDelegate( m_playlistView );
 
     m_playlistView->setFrameShape( QFrame::NoFrame );
     m_playlistView->setContentsMargins( 0, 0, 0, 0 );
@@ -121,6 +123,8 @@ PlaylistBrowserCategory::PlaylistBrowserCategory( int playlistCategory,
     m_playlistView->setAcceptDrops( true );
     m_playlistView->setDropIndicatorShown( true );
 
+    toggleView( toggleAction->isChecked() );
+
     foreach( const Playlists::PlaylistProvider *provider,
              The::playlistManager()->providersForCategory( m_playlistCategory ) )
     {
@@ -177,6 +181,7 @@ PlaylistBrowserCategory::toggleView( bool merged )
         m_playlistView->setModel( m_filterProxy );
         m_playlistView->setItemDelegate( m_defaultItemDelegate );
         m_playlistView->setRootIsDecorated( true );
+        m_addFolderAction->setHelpText( m_addFolderAction->text() );
     }
     else
     {
@@ -184,11 +189,11 @@ PlaylistBrowserCategory::toggleView( bool merged )
         m_playlistView->setModel( m_filterProxy );
         m_playlistView->setItemDelegate( m_byProviderDelegate );
         m_playlistView->setRootIsDecorated( false );
+        m_addFolderAction->setHelpText( i18n( "Folders are only shown in <b>merged view</b>." ) );
     }
 
     //folders don't make sense in per-provider view
     m_addFolderAction->setEnabled( merged );
-    //TODO: set a tooltip saying why it's disabled mention labels
 
     Amarok::config( m_configGroup ).writeEntry( s_mergeViewKey, merged );
 }
@@ -202,8 +207,9 @@ PlaylistBrowserCategory::slotProviderAdded( Playlists::PlaylistProvider *provide
     if( !m_providerActions.keys().contains( provider ) )
         createProviderButton( provider );
 
-    //TODO: only turn off merged view for non-empty providers or based on some provider property
-    toggleView( false );
+    //Hide empty providers when added
+    if( provider->playlistCount() != 0 )
+        toggleView( false );
 
     slotToggleProviderButton();
 }
@@ -237,6 +243,14 @@ PlaylistBrowserCategory::createProviderButton( const Playlists::PlaylistProvider
     else if( m_providerActions.count() == 1 )
         m_providerActions.values().first()->setEnabled( true );
 
+    //Hide empty providers when added
+    if( provider->playlistCount() == 0 )
+    {
+        providerToggle->setChecked( false );
+        //ensure the provider is actually filtered out.
+        slotToggleProviderButton();
+    }
+
     m_providerActions.insert( provider, providerToggle );
 }
 
