@@ -18,7 +18,7 @@
 #define AMAROK_CONTEXT_TOOLBAR_VIEW
 
 #include <QGraphicsView>
-#include <QPointer>
+#include <QWeakPointer>
 
 class QGraphicsScene;
 class QWidget;
@@ -73,7 +73,7 @@ class ToolbarView : public QGraphicsView
     
     private:
         int m_height;
-        QPointer<AppletToolbar> m_toolbar;
+        QWeakPointer<AppletToolbar> m_toolbar;
         QList< AppletItemOverlay* > m_moveOverlays;
         Plasma::Containment* m_cont;
 };    
