@@ -107,7 +107,7 @@ XmlQueryReader::readQuery()
                     d->qm->orderByRandom();
                 else
                 {
-                    qint64 field = fieldVal( fieldStr );
+                    qint64 field = Meta::fieldForName( fieldStr.toString() );
                     bool descending = valueStr == "descending";
 
                     if( field != 0 )
@@ -148,6 +148,18 @@ XmlQueryReader::readQuery()
             {
                 d->qm->setAlbumQueryMode( Collections::QueryMaker::OnlyNormalAlbums );
             }
+            else if( name() == "onlyTrackArtists" )
+            {
+                d->qm->setArtistQueryMode( Collections::QueryMaker::TrackArtists );
+            }
+            else if( name() == "onlyAlbumArtists" )
+            {
+                d->qm->setArtistQueryMode( Collections::QueryMaker::AlbumArtists );
+            }
+            else if( name() == "AlbumOrTrackArtists" )
+            {
+                d->qm->setArtistQueryMode( Collections::QueryMaker::AlbumOrTrackArtists );
+            }
             else if( name() == "returnValues" )
                 readReturnValues();
             //add more container elements here
@@ -198,6 +210,10 @@ XmlQueryReader::readReturnValues()
             {
                 d->qm->setQueryType( Collections::QueryMaker::Album );
             }
+            else if( name() == "albumartist" )
+            {
+                d->qm->setQueryType( Collections::QueryMaker::AlbumArtist );
+            }
             else if( name() == "genres" )
             {
                 d->qm->setQueryType( Collections::QueryMaker::Genre );
@@ -246,7 +262,7 @@ XmlQueryReader::readFilter(QXmlStreamReader *reader)
     QXmlStreamAttributes attr = reader->attributes();
 
     filter.exclude = (reader->name() != "include");
-    filter.field = fieldVal( attr.value( "field" ) );
+    filter.field = Meta::fieldForName( attr.value( "field" ).toString() );
     filter.value = attr.value( "value" ).toString();
 
     QStringRef compareStr = attr.value( "compare" );
@@ -331,34 +347,6 @@ XmlQueryReader::readFilters()
     }
 }
 
-qint64
-XmlQueryReader::fieldVal( QStringRef field )
-{
-    if     ( field == "url"        ) return Meta::valUrl;
-    else if( field == "title"      ) return Meta::valTitle;
-    else if( field == "artist"     ) return Meta::valArtist;
-    else if( field == "album"      ) return Meta::valAlbum;
-    else if( field == "genre"      ) return Meta::valGenre;
-    else if( field == "composer"   ) return Meta::valComposer;
-    else if( field == "year"       ) return Meta::valYear;
-    else if( field == "comment"    ) return Meta::valComment; 
-    else if( field == "tracknr"    ) return Meta::valTrackNr;
-    else if( field == "discnr"     ) return Meta::valDiscNr;
-    else if( field == "length"     ) return Meta::valLength;
-    else if( field == "bitrate"    ) return Meta::valBitrate;
-    else if( field == "samplerate" ) return Meta::valSamplerate;
-    else if( field == "filesize"   ) return Meta::valFilesize;
-    else if( field == "format"     ) return Meta::valFormat;
-    else if( field == "createdate" ) return Meta::valCreateDate;
-    else if( field == "score"      ) return Meta::valScore;
-    else if( field == "rating"     ) return Meta::valRating;
-    else if( field == "firstplay"  ) return Meta::valFirstPlayed;
-    else if( field == "lastplay"   ) return Meta::valLastPlayed;
-    else if( field == "playcount"  ) return Meta::valPlaycount;
-    else if( field == "label"      ) return Meta::valLabel;
-    else                             return 0;
-}
-
 int
 XmlQueryReader::compareVal( QStringRef compare )
 {
