@@ -779,6 +779,17 @@ UmsHandler::libGetArtist( const Meta::MediaDeviceTrackPtr &track )
     return m_umstrackhash.value( track )->artist()->name();
 }
 
+QString
+UmsHandler::libGetAlbumArtist( const Meta::MediaDeviceTrackPtr &track )
+{
+    if ( !m_umstrackhash.contains( track ) )
+        debug() << "Error!  track not in hash!";
+    TrackPtr trackptr = m_umstrackhash.value( track );
+    return ( trackptr->album() && trackptr->album()->hasAlbumArtist() )
+           ? trackptr->album()->albumArtist()->name()
+           : QString();
+}
+
 QString
 UmsHandler::libGetComposer( const Meta::MediaDeviceTrackPtr &track )
 {
@@ -1062,7 +1073,7 @@ UmsHandler::updateTrack( Meta::MediaDeviceTrackPtr &track )
     if ( track->genre() )
         metafile->setGenre( track->genre()->name() );
     if ( track->year() )
-        metafile->setYear( track->year()->name() );
+        metafile->setYear( track->year()->year() );
     metafile->setTrackNumber( track->trackNumber() );
     metafile->setComment( track->comment() );
     metafile->setDiscNumber( track->discNumber() );
