@@ -46,7 +46,6 @@ public:
     virtual ~TimecodeTrack();
 
     virtual QString name() const;
-    virtual QString prettyName() const;
 
     virtual KUrl playableUrl() const;
     virtual QString uidUrl() const;
@@ -65,7 +64,7 @@ public:
     virtual void setArtist ( const QString &newArtist );
     virtual void setGenre ( const QString &newGenre );
     virtual void setComposer ( const QString &newComposer );
-    virtual void setYear ( const QString &newYear );
+    virtual void setYear ( int newYear );
 
     virtual void setTitle( const QString &newTitle );
 
@@ -93,7 +92,6 @@ public:
     virtual int discNumber() const;
     virtual void setDiscNumber ( int newDiscNumber );
 
-    virtual uint lastPlayed() const;
     virtual int playCount() const;
 
     virtual QString type() const;
@@ -161,7 +159,6 @@ public:
     virtual ~TimecodeArtist();
 
     virtual QString name() const;
-    virtual QString prettyName() const;
 
     virtual TrackList tracks();
 
@@ -188,7 +185,6 @@ public:
     virtual ~TimecodeAlbum();
 
     virtual QString name() const;
-    virtual QString prettyName() const;
 
     virtual bool isCompilation() const;
     virtual bool hasAlbumArtist() const;
@@ -197,7 +193,7 @@ public:
 
     virtual QPixmap image( int size = 1 );
     virtual bool canUpdateImage() const;
-    virtual void setImage( const QPixmap &pixmap );
+    virtual void setImage( const QImage &image );
 
     virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const;
     virtual Capabilities::Capability* asCapabilityInterface( Capabilities::Capability::Type type );
@@ -218,7 +214,7 @@ private:
     bool m_isCompilation;
     TimecodeArtistPtr m_albumArtist;
 
-    QPixmap m_cover;
+    QImage m_cover;
     QMap<int, QPixmap> m_coverSizeMap;
 
     QAction *m_separator;
@@ -235,7 +231,6 @@ public:
     virtual ~TimecodeGenre();
 
     virtual QString name() const;
-    virtual QString prettyName() const;
 
     virtual TrackList tracks();
 
@@ -259,7 +254,6 @@ public:
     virtual ~TimecodeComposer();
 
     virtual QString name() const;
-    virtual QString prettyName() const;
 
     virtual TrackList tracks();
 
@@ -283,7 +277,6 @@ public:
     virtual ~TimecodeYear();
 
     virtual QString name() const;
-    virtual QString prettyName() const;
 
     virtual TrackList tracks();
 
