@@ -21,7 +21,7 @@
 #include "core-impl/collections/support/CollectionManager.h"
 #include "core/support/Debug.h"
 #include "core/capabilities/EditCapability.h"
-#include "core/capabilities/CurrentTrackActionsCapability.h"
+#include "core/capabilities/ActionsCapability.h"
 #include "core-impl/capabilities/timecode/TimecodeLoadCapability.h"
 #include "core-impl/capabilities/timecode/TimecodeWriteCapability.h"
 #include "SqlPodcastProvider.h"
@@ -235,7 +235,7 @@ SqlPodcastEpisode::hasCapabilityInterface( Capabilities::Capability::Type type )
 {
     switch( type )
     {
-        case Capabilities::Capability::CurrentTrackActions:
+        case Capabilities::Capability::Actions:
         case Capabilities::Capability::WriteTimecode:
         case Capabilities::Capability::LoadTimecode:
             //only downloaded episodes can be position marked
@@ -255,12 +255,11 @@ SqlPodcastEpisode::createCapabilityInterface( Capabilities::Capability::Type typ
 {
     switch( type )
     {
-        case Capabilities::Capability::CurrentTrackActions:
+        case Capabilities::Capability::Actions:
         {
             QList< QAction * > actions;
-            QAction* flag = new BookmarkCurrentTrackPositionAction( 0 );
-            actions << flag;
-            return new Capabilities::CurrentTrackActionsCapability( actions );
+            actions << new BookmarkCurrentTrackPositionAction( 0 );
+            return new Capabilities::ActionsCapability( actions );
         }
         case Capabilities::Capability::WriteTimecode:
             return new TimecodeWriteCapabilityPodcastImpl( this );
@@ -363,8 +362,8 @@ SqlPodcastEpisode::writeTagsToFile()
     if( m_localFile.isNull() )
         return false;
 
-    Capabilities::EditCapability *ec = m_localFile->create<Capabilities::EditCapability>();
-    if( ec == 0 )
+    QScopedPointer<Capabilities::EditCapability> ec( m_localFile->create<Capabilities::EditCapability>() );
+    if( !ec )
         return false;
 
     debug() << "writing tags for podcast episode " << title() << "to " << m_localUrl.url();
@@ -378,7 +377,7 @@ SqlPodcastEpisode::writeTagsToFile()
     ec->setAlbum( m_channel->title() );
     ec->setArtist( m_channel->author() );
     ec->setGenre( i18n( "Podcast" ) );
-    ec->setYear( QString::number( m_pubDate.date().year() ) );
+    ec->setYear( m_pubDate.date().year() );
     ec->endMetaDataUpdate();
 
     notifyObservers();
@@ -524,7 +523,7 @@ SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
     m_copyright = channel->copyright();
     
     if( channel->hasImage() )
-        m_image = channel->image();
+        m_image = channel->image().toImage();
 
     //Default Settings
 
@@ -593,7 +592,7 @@ SqlPodcastChannel::episodes()
 }
 
 void
-SqlPodcastChannel::setImage( const QPixmap &image )
+SqlPodcastChannel::setImage( const QImage &image )
 {
     DEBUG_BLOCK
 
@@ -609,7 +608,7 @@ SqlPodcastChannel::setImageUrl( const KUrl &imageUrl )
 
     if( imageUrl.isLocalFile() )
     {
-        m_image = QPixmap( imageUrl.path() );
+        m_image = QImage( imageUrl.path() );
         return;
     }
 
