@@ -31,10 +31,12 @@
 #include "core/collections/support/SqlStorage.h"
 #include "statusbar/StatusBar.h"
 #include "SvgHandler.h"
+#include "OpmlWriter.h"
 
 #include "ui_SqlPodcastProviderSettingsWidget.h"
 
 #include <KLocale>
+#include <KFileDialog>
 #include <KIO/CopyJob>
 #include <KIO/DeleteJob>
 #include <KIO/Job>
@@ -60,6 +62,7 @@ SqlPodcastProvider::SqlPodcastProvider()
         , m_updatingChannels( 0 )
         , m_completedDownloads( 0 )
         , m_providerSettingsDialog( 0 )
+        , m_providerSettingsWidget( 0 )
         , m_configureChannelAction( 0 )
         , m_deleteAction( 0 )
         , m_downloadAction( 0 )
@@ -110,17 +113,28 @@ SqlPodcastProvider::SqlPodcastProvider()
         else
             updateDatabase( version /*from*/, PODCAST_DB_VERSION /*to*/ );
 
-        bool startAutoRefreshTimer = false;
-        foreach( Podcasts::SqlPodcastChannelPtr channel, m_channels )
-        {
-            startAutoRefreshTimer = channel->autoScan();
-        }
-        if( startAutoRefreshTimer )
-        {
-            float interval = 1.0;
-            m_updateTimer->start( interval * 1000 * 60 * m_autoUpdateInterval );
-        }
+        startTimer();
+    }
+}
 
+void
+SqlPodcastProvider::startTimer()
+{
+    if( !m_autoUpdateInterval )
+        return; //timer is disabled
+
+    if( m_updateTimer->isActive() &&
+        m_updateTimer->interval() == ( m_autoUpdateInterval * 1000 * 60 ) )
+        return; //already started with correct interval
+
+    //and only start if at least one channel has autoscan enabled
+    foreach( Podcasts::SqlPodcastChannelPtr channel, m_channels )
+    {
+        if( channel->autoScan() )
+        {
+            m_updateTimer->start( 1000 * 60 * m_autoUpdateInterval );
+            return;
+        }
     }
 }
 
@@ -280,6 +294,13 @@ SqlPodcastProvider::providerActions()
         configureAction->setProperty( "popupdropper_svg_id", "configure" );
         connect( configureAction, SIGNAL( triggered() ), this, SLOT( slotConfigureProvider() ) );
         m_providerActions << configureAction;
+
+        QAction *exportOpmlAction = new QAction( KIcon( "document-export" ),
+                                                 i18n( "&Export to subscriptions to OPML file" ),
+                                                 this
+                                               );
+        connect( exportOpmlAction, SIGNAL(triggered()), SLOT(slotExportOpml()) );
+        m_providerActions << exportOpmlAction;
     }
 
     return m_providerActions;
@@ -294,6 +315,7 @@ SqlPodcastProvider::playlistActions( Playlists::PlaylistPtr playlist )
     if( sqlChannel.isNull() )
         return actions;
 
+    //TODO: add export OPML action for selected playlists only. Use the QAction::data() trick.
     if( m_configureChannelAction == 0 )
     {
         m_configureChannelAction = new QAction(
@@ -326,6 +348,7 @@ SqlPodcastProvider::playlistActions( Playlists::PlaylistPtr playlist )
     {
         actionChannels = m_removeAction->data().value<Podcasts::SqlPodcastChannelList>();
     }
+    m_removeAction->setObjectName( "deleteAction" );
 
     actionChannels << sqlChannel;
     m_removeAction->setData( QVariant::fromValue( actionChannels ) );
@@ -382,6 +405,7 @@ SqlPodcastProvider::trackActions( Playlists::PlaylistPtr playlist, int trackInde
         m_deleteAction->setProperty( "popupdropper_svg_id", "delete" );
         connect( m_deleteAction, SIGNAL( triggered() ), SLOT( slotDeleteDownloadedEpisodes() ) );
     }
+    m_deleteAction->setObjectName( "deleteAction" );
 
     if( m_writeTagsAction == 0 )
     {
@@ -509,7 +533,6 @@ SqlPodcastProvider::addChannel( Podcasts::PodcastChannelPtr channel )
 Podcasts::PodcastEpisodePtr
 SqlPodcastProvider::addEpisode( Podcasts::PodcastEpisodePtr episode )
 {
-    DEBUG_BLOCK
     Podcasts::SqlPodcastEpisodePtr sqlEpisode =
             Podcasts::SqlPodcastEpisodePtr::dynamicCast( episode );
     if( sqlEpisode.isNull() )
@@ -563,11 +586,11 @@ SqlPodcastProvider::removeSubscription( Podcasts::SqlPodcastChannelPtr sqlChanne
 void
 SqlPodcastProvider::configureProvider()
 {
-    DEBUG_BLOCK
     m_providerSettingsDialog = new KDialog( The::mainWindow() );
     QWidget *settingsWidget = new QWidget( m_providerSettingsDialog );
     m_providerSettingsDialog->setObjectName( "SqlPodcastProviderSettings" );
     Ui::SqlPodcastProviderSettingsWidget settings;
+    m_providerSettingsWidget = &settings;
     settings.setupUi( settingsWidget );
 
     settings.m_baseDirUrl->setMode( KFile::Directory );
@@ -575,7 +598,7 @@ SqlPodcastProvider::configureProvider()
 
     settings.m_autoUpdateInterval->setValue( m_autoUpdateInterval );
     settings.m_autoUpdateInterval->setPrefix(
-            ki18ncp( "prefix to 'x minutes'", "every ", "every " ).toString() );
+            i18nc( "prefix to 'x minutes'", "every " ) );
     settings.m_autoUpdateInterval->setSuffix( ki18np( " minute", " minutes" ) );
 
     m_providerSettingsDialog->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
@@ -590,33 +613,79 @@ SqlPodcastProvider::configureProvider()
 
     if( m_providerSettingsDialog->exec() == QDialog::Accepted )
     {
-        debug() << "accepted";
-
-        //TODO: apply
+        m_autoUpdateInterval = settings.m_autoUpdateInterval->value();
+        if( m_autoUpdateInterval )
+            startTimer();
+        else
+            m_updateTimer->stop();
     }
 
     delete m_providerSettingsDialog;
     m_providerSettingsDialog = 0;
-
+    m_providerSettingsWidget = 0;
 }
 
 void
 SqlPodcastProvider::slotConfigChanged()
 {
-    Ui::SqlPodcastProviderSettingsWidget *settings =
-            dynamic_cast<Ui::SqlPodcastProviderSettingsWidget *>(
-                    m_providerSettingsDialog->mainWidget() );
-
-    if( !settings )
+    if( !m_providerSettingsWidget )
         return;
 
-    if( settings->m_autoUpdateInterval->value() != m_autoUpdateInterval
-        || settings->m_baseDirUrl->url() != m_baseDownloadDir )
+    if( m_providerSettingsWidget->m_autoUpdateInterval->value() != m_autoUpdateInterval
+        || m_providerSettingsWidget->m_baseDirUrl->url() != m_baseDownloadDir )
     {
         m_providerSettingsDialog->enableButtonApply( true );
     }
 }
 
+void
+SqlPodcastProvider::slotExportOpml()
+{
+    OpmlOutline *rootOutline = new OpmlOutline();
+    //TODO: root OPML outline head
+
+    //TODO: folder outline support
+    foreach( SqlPodcastChannelPtr channel, m_channels )
+    {
+        OpmlOutline *channelOutline = new OpmlOutline( rootOutline );
+        #define addAttr( k, v ) channelOutline->addAttribute( k, v )
+        addAttr( "text", channel->title() );
+        addAttr( "type", "rss" );
+        addAttr( "xmlUrl", channel->url().url() );
+        rootOutline->addChild( channelOutline );
+    }
+
+    //TODO: add checkbox as widget to filedialog to include podcast settings.
+    KFileDialog fileDialog( KUrl( "kfiledialog:///podcast/amarok_podcasts.opml"), "*.opml",
+                            The::mainWindow() );
+    fileDialog.setMode( KFile::File );
+    fileDialog.setCaption( i18n( "Select file for OPML export") );
+    if( fileDialog.exec() != KDialog::Accepted )
+        return;
+
+    KUrl filePath = fileDialog.selectedUrl();
+
+    QFile *opmlFile = new QFile( filePath.toLocalFile(), this );
+    if( !opmlFile->open( QIODevice::WriteOnly | QIODevice::Truncate ) )
+    {
+        error() << "could not open OPML file " << filePath.url();
+        return;
+    }
+    OpmlWriter *opmlWriter = new OpmlWriter( rootOutline, opmlFile );
+    connect( opmlWriter, SIGNAL(result(int)), SLOT(slotOpmlWriterDone(int)) );
+    opmlWriter->run();
+}
+
+void
+SqlPodcastProvider::slotOpmlWriterDone( int result )
+{
+    Q_UNUSED( result )
+    OpmlWriter *writer = qobject_cast<OpmlWriter *>( QObject::sender() );
+    Q_ASSERT( writer );
+    writer->device()->close();
+    delete writer;
+}
+
 void
 SqlPodcastProvider::configureChannel( Podcasts::SqlPodcastChannelPtr sqlChannel )
 {
@@ -627,6 +696,7 @@ SqlPodcastProvider::configureChannel( Podcasts::SqlPodcastChannelPtr sqlChannel
     KUrl oldSaveLocation = sqlChannel->saveLocation();
     bool oldHasPurge = sqlChannel->hasPurge();
     int oldPurgeCount = sqlChannel->purgeCount();
+    bool oldAutoScan = sqlChannel->autoScan();
 
     PodcastSettingsDialog dialog( sqlChannel, The::mainWindow() );
     dialog.configure();
@@ -690,6 +760,10 @@ SqlPodcastProvider::configureChannel( Podcasts::SqlPodcastChannelPtr sqlChannel
     //if the url changed force an update.
     if( oldUrl != sqlChannel->url() )
         updateSqlChannel( sqlChannel );
+
+    //start autoscan in case it wasn't already
+    if( sqlChannel->autoScan() && !oldAutoScan )
+        startTimer();
 }
 
 QList<QAction *>
@@ -1045,7 +1119,6 @@ SqlPodcastProvider::completePodcastDownloads()
 void
 SqlPodcastProvider::autoUpdate()
 {
-    DEBUG_BLOCK
     if( Solid::Networking::status() != Solid::Networking::Connected
             && Solid::Networking::status() != Solid::Networking::Unknown )
     {
@@ -1091,7 +1164,6 @@ SqlPodcastProvider::updateSqlChannel( Podcasts::SqlPodcastChannelPtr channel )
 void
 SqlPodcastProvider::slotReadResult( Podcasts::PodcastReader *podcastReader )
 {
-    DEBUG_BLOCK
     if( podcastReader->error() != QXmlStreamReader::NoError )
     {
         debug() << podcastReader->errorString();
@@ -1208,8 +1280,6 @@ SqlPodcastProvider::downloadEpisode( Podcasts::PodcastEpisodePtr episode )
 void
 SqlPodcastProvider::cleanupDownload( KJob *job, bool downloadFailed )
 {
-    DEBUG_BLOCK
-
     QFile *tmpFile = m_tmpFileMap.value( job );
 
     if( downloadFailed && tmpFile )
@@ -1227,8 +1297,6 @@ SqlPodcastProvider::cleanupDownload( KJob *job, bool downloadFailed )
 QFile*
 SqlPodcastProvider::createTmpFile( KJob *job )
 {
-    DEBUG_BLOCK
-
     Podcasts::SqlPodcastEpisodePtr sqlEpisode = m_downloadJobMap.value( job );
     if( sqlEpisode.isNull() )
     {
@@ -1337,7 +1405,6 @@ SqlPodcastProvider::addData( KIO::Job *job, const QByteArray &data )
 void
 SqlPodcastProvider::deleteDownloadedEpisode( Podcasts::PodcastEpisodePtr episode )
 {
-    DEBUG_BLOCK
     deleteDownloadedEpisode( SqlPodcastEpisodePtr::dynamicCast( episode ) );
 }
 
@@ -1434,8 +1501,6 @@ SqlPodcastProvider::redirected( KIO::Job *job, const KUrl & redirectedUrl )
 void
 SqlPodcastProvider::createTables() const
 {
-    DEBUG_BLOCK
-
     SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
     if( !sqlStorage )
         return;
@@ -1568,14 +1633,12 @@ SqlPodcastProvider::updateDatabase( int fromVersion, int toVersion )
 void
 SqlPodcastProvider::fetchImage( SqlPodcastChannelPtr channel )
 {
-    DEBUG_BLOCK
-
     if( m_podcastImageFetcher == 0 )
     {
         m_podcastImageFetcher = new PodcastImageFetcher();
         connect( m_podcastImageFetcher,
-                 SIGNAL( imageReady( Podcasts::PodcastChannelPtr, QPixmap ) ),
-                 SLOT( channelImageReady( Podcasts::PodcastChannelPtr, QPixmap ) )
+                 SIGNAL( imageReady( Podcasts::PodcastChannelPtr, QImage ) ),
+                 SLOT( channelImageReady( Podcasts::PodcastChannelPtr, QImage ) )
                );
         connect( m_podcastImageFetcher,
                  SIGNAL( done( PodcastImageFetcher * ) ),
@@ -1587,14 +1650,12 @@ SqlPodcastProvider::fetchImage( SqlPodcastChannelPtr channel )
 }
 
 void
-SqlPodcastProvider::channelImageReady( Podcasts::PodcastChannelPtr channel, QPixmap pixmap )
+SqlPodcastProvider::channelImageReady( Podcasts::PodcastChannelPtr channel, QImage image )
 {
-    DEBUG_BLOCK
-    debug() << "channel: " << channel->title();
-    if( pixmap.isNull() )
+    if( image.isNull() )
         return;
 
-    channel->setImage( pixmap );
+    channel->setImage( image );
 }
 
 void
