@@ -32,6 +32,10 @@ class PodcastReader;
 class SqlStorage;
 class QTimer;
 
+namespace Ui {
+    class SqlPodcastProviderSettingsWidget;
+}
+
 namespace Podcasts {
 
 /**
@@ -100,12 +104,13 @@ class SqlPodcastProvider : public Podcasts::PodcastProvider
         void slotDownloadProgress( KJob *job, unsigned long percent );
         void slotWriteTagsToFiles();
         void slotConfigChanged();
+        void slotExportOpml();
 
     signals:
         void totalPodcastDownloadProgress( int progress );
 
     private slots:
-        void channelImageReady( Podcasts::PodcastChannelPtr, QPixmap );
+        void channelImageReady( Podcasts::PodcastChannelPtr, QImage );
         void podcastImageFetcherDone( PodcastImageFetcher * );
         void slotConfigureProvider();
 
@@ -113,8 +118,10 @@ class SqlPodcastProvider : public Podcasts::PodcastProvider
                                                                const QString &description,
                                                                Podcasts::PodcastReader* reader );
         void slotStatusBarSorryMessage( const QString &message );
+        void slotOpmlWriterDone( int result );
 
     private:
+        void startTimer();
         void configureProvider();
         void configureChannel( Podcasts::SqlPodcastChannelPtr channel );
         void updateSqlChannel( Podcasts::SqlPodcastChannelPtr channel );
@@ -168,6 +175,7 @@ class SqlPodcastProvider : public Podcasts::PodcastProvider
         KUrl m_baseDownloadDir;
 
         KDialog *m_providerSettingsDialog;
+        Ui::SqlPodcastProviderSettingsWidget *m_providerSettingsWidget;
 
         QList<QAction *> m_providerActions;
 
