@@ -1,6 +1,7 @@
 /****************************************************************************************
  * Copyright (c) 2003-2005 Max Howell <max.howell@methylblue.com>                       *
  * Copyright (c) 2007-2009 Mark Kretschmann <kretschmann@kde.org>                       *
+ * Copyright (c) 2010 Kevin Funk <krf@electrostorm.net>                                 *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -22,25 +23,32 @@
 #undef QT_NO_DEBUG_OUTPUT
 #undef KDE_NO_DEBUG_OUTPUT
 
-#include <KGlobal>
-#include <KCmdLineArgs>
-#include <KConfig>
-#include <KConfigGroup>
-#include <kdebug.h>
-#include <QApplication>
-#include <QMutex>
-#include <QObject>
-
-#include <sys/time.h>
-#include <unistd.h>
-#include <iostream>
-#include <cerrno>
-
 #include "shared/amarok_export.h"
 
+#include <kdebug.h>
+
+#include <QMutex>
+
+#if QT_VERSION >= 0x040700
+# include <QElapsedTimer>
+#else
+# include <QTime>
+#endif
+
+// Platform specific macros
 #ifdef _WIN32
 #define __PRETTY_FUNCTION__ __FUNCTION__
 #endif
+#ifdef __SUNPRO_CC
+#define __PRETTY_FUNCTION__ __FILE__
+#endif
+
+// Debug prefix, override if needed
+#ifndef DEBUG_PREFIX
+#define AMAROK_PREFIX ""
+#else
+#define AMAROK_PREFIX "[" DEBUG_PREFIX "]"
+#endif
 
 /**
  * @namespace Debug
@@ -71,83 +79,39 @@
  * @see CrashHelper
  * @see ListStream
  */
-
 namespace Debug
 {
     extern AMAROK_CORE_EXPORT QMutex mutex;
 
-    // we can't use a statically instantiated QString for the indent, because
-    // static namespaces are unique to each dlopened library. So we piggy back
-    // the QString on the KApplication instance
-
-    #define qOApp reinterpret_cast<QObject*>(qApp)
-    class Indent : QObject
-    {
-        friend QString &modifieableIndent();
-        Indent() : QObject( qOApp ) { setObjectName( "DEBUG_indent" ); }
-        QString m_string;
-    };
-
-    inline QString &modifieableIndent()
-    {
-        QObject* o = qOApp ? qOApp->findChild<QObject*>( "DEBUG_indent" ) : 0;
-        QString &ret = (o ? static_cast<Indent*>( o ) : new Indent)->m_string;
-        return ret;
-    }
-
-    inline QString indent()
-    {
-        return modifieableIndent();
-    }
-
-    inline bool debugEnabled()
-    {
-        KConfigGroup config = KGlobal::config()->group( "General" );
-        const bool debug = config.readEntry( "Debug Enabled", false );
-        
-        return debug; 
-    }
-
-    inline kdbgstream dbgstream()
-    {
-        return debugEnabled() ? kdbgstream( QtDebugMsg ) : kDebugDevNull();
-    }
- 
-    #undef qOApp
-
-    #ifndef DEBUG_PREFIX
-    #define AMK_PREFIX ""
-    #else
-    #define AMK_PREFIX "[" DEBUG_PREFIX "]"
-    #endif
-
-    //from kdebug.h
-    enum DebugLevels {
+    // from kdebug.h
+    enum DebugLevel {
         KDEBUG_INFO  = 0,
         KDEBUG_WARN  = 1,
         KDEBUG_ERROR = 2,
         KDEBUG_FATAL = 3
     };
 
-        
-    static inline kdbgstream debug()   { mutex.lock(); QString ind = indent(); mutex.unlock(); return dbgstream() << qPrintable( "amarok: " + ind + AMK_PREFIX ); }
-    static inline kdbgstream warning() { mutex.lock(); QString ind = indent(); mutex.unlock(); return dbgstream() << qPrintable( "amarok: " + ind + AMK_PREFIX + " [WARNING!]" ); }
-    static inline kdbgstream error()   { mutex.lock(); QString ind = indent(); mutex.unlock(); return dbgstream() << qPrintable( "amarok: " + ind + AMK_PREFIX + " [ERROR!]" ); }
-    static inline kdbgstream fatal()   { mutex.lock(); QString ind = indent(); mutex.unlock(); return dbgstream() << qPrintable( "amarok: " + ind + AMK_PREFIX ); }
+    AMAROK_CORE_EXPORT kdbgstream dbgstream( DebugLevel level = KDEBUG_INFO );
+    AMAROK_CORE_EXPORT bool debugEnabled();
+    AMAROK_CORE_EXPORT bool debugColorEnabled();
+    AMAROK_CORE_EXPORT void setDebugEnabled( bool enable );
+    AMAROK_CORE_EXPORT void setColoredDebug( bool enable );
+    AMAROK_CORE_EXPORT QString indent();
 
-    #undef AMK_PREFIX
-
-    static inline void perfLog( const QString &message, const QString &func )
-    {
-#ifdef Q_OS_UNIX
-        if( !debugEnabled() )
-        {
-            return;
-        }
-        QString str = QString( "MARK: %1: %2 %3" ).arg( KCmdLineArgs::appName(), func, message );
-        access( str.toLocal8Bit().data(), F_OK );
+    static inline kdbgstream dbgstreamwrapper( DebugLevel level ) {
+#ifdef DEBUG_PREFIX
+        return dbgstream( level ) << AMAROK_PREFIX;
+#else
+        return dbgstream( level );
 #endif
     }
+
+    static inline kdbgstream debug()   { return dbgstreamwrapper( KDEBUG_INFO ); }
+    static inline kdbgstream warning() { return dbgstreamwrapper( KDEBUG_WARN ); }
+    static inline kdbgstream error()   { return dbgstreamwrapper( KDEBUG_ERROR ); }
+    static inline kdbgstream fatal()   { return dbgstreamwrapper( KDEBUG_FATAL ); }
+
+    AMAROK_CORE_EXPORT void perfLog( const QString &message, const QString &func );
 }
 
 using Debug::debug;
@@ -161,22 +125,20 @@ using Debug::fatal;
 /// Announce a line
 #define DEBUG_LINE_INFO { Debug::mutex.lock(); kDebug() << Debug::indent() << "Line: " << __LINE__; Debug::mutex.unlock(); }
 
-#ifdef __SUNPRO_CC
-#define __PRETTY_FUNCTION__ __FILE__
-#endif
-
 /// Convenience macro for making a standard Debug::Block
 #define DEBUG_BLOCK Debug::Block uniquelyNamedStackAllocatedStandardBlock( __PRETTY_FUNCTION__ );
 
 /// Use this to remind yourself to finish the implementation of a function
-#define AMAROK_NOTIMPLEMENTED warning() << "NOT-IMPLEMENTED: " << __PRETTY_FUNCTION__ << endl;
+#define AMAROK_NOTIMPLEMENTED warning() << "NOT-IMPLEMENTED:" << __PRETTY_FUNCTION__ << endl;
 
 /// Use this to alert other developers to stop using a function
-#define AMAROK_DEPRECATED warning() << "DEPRECATED: " << __PRETTY_FUNCTION__ << endl;
+#define AMAROK_DEPRECATED warning() << "DEPRECATED:" << __PRETTY_FUNCTION__ << endl;
 
 /// Performance logging
 #define PERF_LOG( msg ) { Debug::perfLog( msg, __PRETTY_FUNCTION__ ); }
 
+class BlockPrivate;
+
 namespace Debug
 {
     /**
@@ -201,65 +163,22 @@ namespace Debug
      *     app: END: section - Took 0.1s
      *
      */
-
     class Block
     {
-        timeval     m_start;
-        const char *m_label;
-
     public:
-        Block( const char *label )
-                : m_label( label )
-        {
-            if ( gettimeofday( &m_start, 0 ) == -1 )
-                dbgstream() << "amarok: Block - gettimeofday failed with "
-                            << strerror(errno);
+        AMAROK_CORE_EXPORT Block( const char *name );
+        AMAROK_CORE_EXPORT ~Block();
 
-            if( !debugEnabled() ) return;
-
-            mutex.lock();
-
-            dbgstream() << qPrintable( "amarok: " + indent() + "BEGIN: " + label );
-            Debug::modifieableIndent() += "  ";
-            mutex.unlock();
-        }
-
-        ~Block()
-        {
-            if( !debugEnabled() ) return;
-
-            mutex.lock();
-            timeval end;
-            int result = gettimeofday( &end, 0 );
-            if( result == -1 )
-            {
-                mutex.unlock();
-                return;
-            }
-
-            end.tv_sec -= m_start.tv_sec;
-            if( end.tv_usec < m_start.tv_usec) {
-                // Manually carry a one from the seconds field.
-                end.tv_usec += 1000000;
-                end.tv_sec--;
-            }
-            end.tv_usec -= m_start.tv_usec;
-
-            double duration = double(end.tv_sec) + (double(end.tv_usec) / 1000000.0);
-
-            Debug::modifieableIndent().truncate( Debug::indent().length() - 2 );
-
-            // Print timing information, and a special message (DELAY) if the method took longer than 5s
-            if( duration < 5.0 )
-                dbgstream() << qPrintable( "amarok: " + indent() + "END__: " + m_label + " - Took " + QString::number( duration, 'g', 2 ) + "s" );
-            else
-                dbgstream() << qPrintable( "amarok: " + indent() + "END__: " + m_label + " - DELAY Took (quite long) " + QString::number( duration, 'g', 2 ) + "s" );
-
-            mutex.unlock();
-        }
+    private:
+#if QT_VERSION >= 0x040700
+        QElapsedTimer m_startTime;
+#else
+        QTime m_startTime;
+#endif
+        const char *m_label;
+        int m_color;
     };
 
-
     /**
      * @name Debug::stamp()
      * @short To facilitate crash/freeze bugs, by making it easy to mark code that has been processed
@@ -280,15 +199,9 @@ namespace Debug
      *     app: Stamp: 2
      *
      */
-
-    inline void stamp()
-    {
-        static int n = 0;
-        debug() << "| Stamp: " << ++n << endl;
-    }
+    AMAROK_CORE_EXPORT void stamp();
 }
 
-
 #include <QVariant>
 
 namespace Debug
