@@ -28,7 +28,7 @@
 
 #include <QMetaMethod>
 #include <QNetworkReply>
-#include <QPointer>
+#include <QWeakPointer>
 #include <QWeakPointer>
 
 NetworkAccessManagerProxy *NetworkAccessManagerProxy::s_instance = 0;
@@ -67,28 +67,28 @@ public:
         Q_Q( NetworkAccessManagerProxy );
         QNetworkReply *reply = static_cast<QNetworkReply*>( q->sender() );
         KUrl url = reply->request().url();
-        QList<CallBackData> callbacks = urlMap.values( url );
+        QList<CallBackData*> callbacks = urlMap.values( url );
         QByteArray data = reply->readAll();
         data.detach(); // detach so the bytes are not deleted before methods are invoked
-        foreach( const CallBackData &cb, callbacks )
+        foreach( const CallBackData *cb, callbacks )
         {
-            QByteArray sig = QMetaObject::normalizedSignature( cb.method );
+            QByteArray sig = QMetaObject::normalizedSignature( cb->method );
             sig.remove( 0, 1 ); // remove first char, which is the member code (see qobjectdefs.h)
                                 // and let Qt's meta object system handle the rest.
-            if( !cb.receiver.isNull() )
+            if( cb->receiver )
             {
                 bool success( false );
-                const QMetaObject *mo = cb.receiver.data()->metaObject();
+                const QMetaObject *mo = cb->receiver.data()->metaObject();
                 int methodIndex = mo->indexOfSlot( sig );
                 if( methodIndex != -1 )
                 {
                     Error err = { reply->error(), reply->errorString() };
                     QMetaMethod method = mo->method( methodIndex );
-                    success = method.invoke( cb.receiver.data(),
-                                             cb.type,
+                    success = method.invoke( cb->receiver.data(),
+                                             cb->type,
                                              Q_ARG( KUrl, reply->request().url() ),
                                              Q_ARG( QByteArray, data ),
-                                             Q_ARG( Error, err ) );
+                                             Q_ARG( NetworkAccessManagerProxy::Error, err ) );
                 }
 
                 if( !success )
@@ -98,22 +98,34 @@ public:
                 }
             }
         }
+        qDeleteAll( callbacks );
         urlMap.remove( url );
         reply->deleteLater();
     }
 
-    struct CallBackData
+    class CallBackData
     {
-#if QT_VERSION >= 0x040600
+    public:
+        CallBackData( QObject *rec, QNetworkReply *rep, const char *met, Qt::ConnectionType t )
+            : receiver( rec )
+            , reply( rep )
+            , method( met )
+            , type( t )
+        {}
+
+        ~CallBackData()
+        {
+            if( reply )
+                reply.data()->deleteLater();
+        }
+
         QWeakPointer<QObject> receiver;
-#else
-        QPointer<QObject> receiver;
-#endif
+        QWeakPointer<QNetworkReply> reply;
         const char *method;
         Qt::ConnectionType type;
     };
 
-    QMultiHash<KUrl, CallBackData> urlMap;
+    QMultiHash<KUrl, CallBackData*> urlMap;
     QString userAgent;
 #ifdef DEBUG_BUILD_TYPE
     NetworkAccessViewer *viewer;
@@ -125,11 +137,7 @@ private:
 };
 
 NetworkAccessManagerProxy::NetworkAccessManagerProxy( QObject *parent )
-#if KDE_IS_VERSION(4, 4, 0)
     : KIO::Integration::AccessManager( parent )
-#else
-    : KIO::AccessManager( parent )
-#endif
     , d( new NetworkAccessManagerProxyPrivate( this ) )
 {
     setCache(0);   // disable QtWebKit cache to just use KIO one..
@@ -172,17 +180,33 @@ NetworkAccessManagerProxy::getData( const KUrl &url, QObject *receiver, const ch
         return 0;
     }
 
-    NetworkAccessManagerProxyPrivate::CallBackData cbm = { receiver, method, type };
-    if( d->urlMap.contains(url) )
-    {
-        d->urlMap.insert( url, cbm );
-        return 0;
-    }
-
-    QNetworkReply *reply = get( QNetworkRequest(url) );
+    QNetworkReply *r = d->urlMap.contains(url) ? d->urlMap.value(url)->reply.data() : get( QNetworkRequest(url) );
+    typedef NetworkAccessManagerProxyPrivate::CallBackData PrivateCallBackData;
+    PrivateCallBackData *cbm = new PrivateCallBackData( receiver, r, method, type );
     d->urlMap.insert( url, cbm );
-    connect( reply, SIGNAL(finished()), this, SLOT(_replyFinished()), type );
-    return reply;
+    connect( r, SIGNAL(finished()), this, SLOT(_replyFinished()), type );
+    return r;
+}
+
+int
+NetworkAccessManagerProxy::abortGet( const KUrl::List &urls )
+{
+    int removed = 0;
+    const QSet<KUrl> &urlSet = urls.toSet();
+    foreach( const KUrl &url, urlSet )
+        removed += abortGet( url );
+    return removed;
+}
+
+int
+NetworkAccessManagerProxy::abortGet( const KUrl &url )
+{
+    if( !d->urlMap.contains(url) )
+        return 0;
+
+    qDeleteAll( d->urlMap.values( url ) );
+    int removed = d->urlMap.remove( url );
+    return removed;
 }
 
 void
@@ -200,9 +224,7 @@ QNetworkReply *
 NetworkAccessManagerProxy::createRequest( Operation op, const QNetworkRequest &req, QIODevice *outgoingData )
 {
     QNetworkRequest request = req;
-#if QT_VERSION >= 0x040600
     request.setAttribute( QNetworkRequest::HttpPipeliningAllowedAttribute, true );
-#endif
     request.setRawHeader( "User-Agent", d->userAgent.toLocal8Bit() );
 
     KIO::CacheControl cc = KProtocolManager::cacheControl();
@@ -227,11 +249,7 @@ NetworkAccessManagerProxy::createRequest( Operation op, const QNetworkRequest &r
         break;
     }
 
-#if KDE_IS_VERSION(4, 4, 0)
     QNetworkReply *reply = KIO::Integration::AccessManager::createRequest( op, request, outgoingData );
-#else
-    QNetworkReply *reply = KIO::AccessManager::createRequest( op, request, outgoingData );
-#endif
 
 #ifdef DEBUG_BUILD_TYPE
     if( d->viewer )
