@@ -22,6 +22,7 @@
 
 #include <kdeversion.h>
 #include <KIO/AccessManager>
+#include <KUrl>
 
 #include <QNetworkReply>
 
@@ -35,11 +36,7 @@ namespace The
     AMAROK_EXPORT NetworkAccessManagerProxy *networkAccessManager();
 }
 
-#if KDE_IS_VERSION(4, 4, 0)
 class AMAROK_EXPORT NetworkAccessManagerProxy : public KIO::Integration::AccessManager
-#else
-class AMAROK_EXPORT NetworkAccessManagerProxy : public KIO::AccessManager
-#endif
 {
     Q_OBJECT
 
@@ -68,6 +65,9 @@ public:
     QNetworkReply *getData( const KUrl &url, QObject *receiver, const char *method,
                             Qt::ConnectionType type = Qt::AutoConnection );
 
+    int abortGet( const KUrl &url );
+    int abortGet( const KUrl::List &urls );
+
 #ifdef DEBUG_BUILD_TYPE
     NetworkAccessViewer *networkAccessViewer();
     void setNetworkAccessViewer( NetworkAccessViewer *viewer );
