@@ -117,7 +117,7 @@ Playlist::Controller::insertOptioned( Meta::TrackList list, int options )
 
         topModelInsertRow = m_topModel->activeRow() + 1;
 
-        while( m_topModel->stateOfRow( topModelInsertRow ) & Item::Queued )
+        while( m_topModel->queuePositionOfRow( topModelInsertRow ) )
             topModelInsertRow++;    // We want to add the newly queued items after any items which are already queued
 
         int bottomModelInsertRow = insertionTopRowToBottom( topModelInsertRow );
@@ -141,17 +141,20 @@ Playlist::Controller::insertOptioned( Meta::TrackList list, int options )
         insertionHelper( insertionTopRowToBottom( topModelInsertRow ), list );
     }
 
-    const Phonon::State engineState = The::engineController()->state();
-    debug() << "engine state: " << engineState;
+    debug() << "engine playing?: " << The::engineController()->isPlaying();
 
     bool playNow = false;
-    if ( options & DirectPlay )
+    if( options & DirectPlay )
         playNow = true;
-    if ( options & StartPlay )
-        if ( ( engineState == Phonon::StoppedState ) || ( engineState == Phonon::LoadingState ) || ( engineState == Phonon::PausedState) )
-            playNow = true;
 
-    if ( playNow ) {
+    if( options & StartPlay )
+    {
+        if( !The::engineController()->isPlaying() && !The::engineController()->isPaused() )
+            playNow = true;
+    }
+
+    if( playNow )
+    {
         if ( AmarokConfig::trackProgression() == AmarokConfig::EnumTrackProgression::RandomTrack ||
              AmarokConfig::trackProgression() == AmarokConfig::EnumTrackProgression::RandomAlbum )
             Actions::instance()->play();
