@@ -17,6 +17,8 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "PlaylistDock"
+
 #include "PlaylistDock.h"
 
 #include "ActionClasses.h"
@@ -32,6 +34,7 @@
 #include "PlaylistDefines.h"
 #include "PlaylistManager.h"
 #include "PlaylistModelStack.h"
+#include "PlaylistQueueEditor.h"
 #include "ProgressiveSearchWidget.h"
 #include "core-impl/playlists/providers/user/UserPlaylistProvider.h"
 #include "widgets/HorizontalDivider.h"
@@ -42,7 +45,7 @@
 #include <QHBoxLayout>
 
 Playlist::Dock::Dock( QWidget* parent )
-    : AmarokDockWidget( i18n( "Playlist" ), parent )
+    : AmarokDockWidget( i18n( "&Playlist" ), parent )
 {
     DEBUG_BLOCK
 
@@ -50,6 +53,27 @@ Playlist::Dock::Dock( QWidget* parent )
     setAllowedAreas( Qt::AllDockWidgetAreas );
 }
 
+Playlist::PrettyListView *
+Playlist::Dock::currentView()
+{
+    ensurePolish();
+    return m_playlistView;
+}
+
+Playlist::SortWidget *
+Playlist::Dock::sortWidget()
+{
+    ensurePolish();
+    return m_sortWidget;
+}
+
+Playlist::ProgressiveSearchWidget *
+Playlist::Dock::searchWidget()
+{
+    ensurePolish();
+    return m_searchWidget;
+}
+
 void
 Playlist::Dock::polish()
 {
@@ -88,7 +112,7 @@ Playlist::Dock::polish()
     QVBoxLayout* mainPlaylistlayout = new QVBoxLayout( layoutHolder );
     mainPlaylistlayout->setContentsMargins( 0, 0, 0, 0 );
 
-    m_playlistView = new PrettyListView( m_mainWidget );
+    m_playlistView = new PrettyListView();
     m_playlistView->show();
 
     connect( m_searchWidget, SIGNAL( filterChanged( const QString &, int, bool ) ), m_playlistView, SLOT( find( const QString &, int, bool ) ) );
@@ -97,7 +121,8 @@ Playlist::Dock::polish()
     connect( m_searchWidget, SIGNAL( filterCleared() ), m_playlistView, SLOT( clearSearchTerm() ) );
     connect( m_searchWidget, SIGNAL( showOnlyMatches( bool ) ), m_playlistView, SLOT( showOnlyMatches( bool ) ) );
     connect( m_searchWidget, SIGNAL( activateFilterResult() ), m_playlistView, SLOT( playFirstSelected() ) );
-    connect( m_searchWidget, SIGNAL( downPressed() ), m_playlistView, SLOT( setFocus() ) );
+    connect( m_searchWidget, SIGNAL( downPressed() ), m_playlistView, SLOT( downOneTrack() ) );
+    connect( m_searchWidget, SIGNAL( upPressed() ), m_playlistView, SLOT( upOneTrack() ) );
 
     connect( The::mainWindow(), SIGNAL( switchQueueStateShortcut() ), m_playlistView, SLOT( switchQueueState() ) );
 
@@ -163,11 +188,17 @@ Playlist::Dock::polish()
 
         NavigatorConfigAction * navigatorConfig = new NavigatorConfigAction( m_mainWidget );
         plBar->addAction( navigatorConfig );
-        plBar->addAction( new KToolBarSpacerAction( m_mainWidget ) );
+        plBar->addSeparator();
 
         QToolButton *toolButton = qobject_cast<QToolButton*>(plBar->widgetForAction( navigatorConfig ) );
         if( toolButton )
             toolButton->setPopupMode( QToolButton::InstantPopup );
+
+        QAction *queueEditAction = Amarok::actionCollection()->action( "playlist_edit_queue" );
+        plBar->addAction( queueEditAction );
+        plBar->addAction( new KToolBarSpacerAction( m_mainWidget ) );
+        connect( queueEditAction, SIGNAL( triggered( bool ) ),
+                 SLOT( slotEditQueue() ) );
     } // END Playlist Toolbar
 
     // If it is active, clear the search filter before replacing the playlist. Fixes Bug #200709.
@@ -203,7 +234,7 @@ Playlist::Dock::playlistProviderAdded( Playlists::PlaylistProvider *provider, in
     if( userProvider == 0 )
         return;
     QAction *action = new KAction( userProvider->icon(), i18n("&Save playlist to \"%1\"", provider->prettyName() ), this );
-    action->setData( QVariant::fromValue( QPointer<Playlists::UserPlaylistProvider>( userProvider ) ) );
+    action->setData( QVariant::fromValue( QWeakPointer<Playlists::UserPlaylistProvider>( userProvider ) ) );
     m_saveActions->addAction( provider->objectName(), action );
 
     m_savePlaylistMenu->addAction( action );
@@ -230,10 +261,23 @@ Playlist::Dock::slotSaveCurrentPlaylist()
     if( action == 0 )
         return;
 
-    Playlists::UserPlaylistProvider *provider =
-            action->data().value< QPointer<Playlists::UserPlaylistProvider> >();
+    QWeakPointer<Playlists::UserPlaylistProvider> pointer = action->data().value< QWeakPointer<Playlists::UserPlaylistProvider> >();
+    Playlists::UserPlaylistProvider* provider = pointer.data();
 
-    The::playlistManager()->save( The::playlist()->tracks(), QString(), provider );
+    The::playlistManager()->save( The::playlist()->tracks(),
+                                  Playlist::ModelStack::instance()->bottom()->generatePlaylistName(), provider );
+}
+
+void
+Playlist::Dock::slotEditQueue()
+{
+    if( m_playlistQueueEditor ) {
+        m_playlistQueueEditor.data()->raise();
+        return;
+    }
+    m_playlistQueueEditor = new PlaylistQueueEditor;
+    m_playlistQueueEditor.data()->setAttribute( Qt::WA_DeleteOnClose );
+    m_playlistQueueEditor.data()->show();
 }
 
 void
