@@ -4,6 +4,7 @@
  * Copyright (c) 2008 Seb Ruiz <ruiz@kde.org>                                           *
  * Copyright (c) 2008 Soren Harward <stharward@gmail.com>                               *
  * Copyright (c) 2010 Nanno Langstraat <langstr@gmail.com>                              *
+ * Copyright (c) 2010 Dennis Francis <dennisfrancis.in@gmail.com>                       *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -35,6 +36,7 @@
 #include "core/capabilities/SourceInfoCapability.h"
 #include "core/collections/Collection.h"
 #include "core/meta/support/MetaUtility.h"
+#include "PlaylistColumnNames.h"
 #include "PlaylistActions.h"
 #include "PlaylistModelStack.h"
 #include "PlaylistItem.h"
@@ -43,6 +45,7 @@
 
 #include <KGlobal>
 #include <KUrl>
+#include <KIconLoader>
 
 #include <QAction>
 #include <QDate>
@@ -51,6 +54,99 @@
 
 #include <typeinfo>
 
+bool Playlist::Model::s_tooltipColumns[NUM_COLUMNS];
+
+// ------- helper functions for the tooltip
+
+static QString
+breakLongLinesHTML(const QString& text)
+{
+    // Now let's break up long lines so that the tooltip doesn't become hideously large
+
+    // The size of the normal, standard line
+    const int lnSize = 50;
+    if (text.size() <= lnSize)
+    {
+        // If the text is not too long, return it as it is
+        return text;
+    }
+    else
+    {
+        QString textInLines;
+
+        QStringList words = text.trimmed().split(' ');
+        int lineLength = 0;
+        while(words.size() > 0)
+        {
+            QString word = words.first();
+            // Let's check if the next word makes the current line too long.
+            if (lineLength + word.size() + 1 > lnSize)
+            {
+                if (lineLength > 0)
+                {
+                    textInLines += "<br/>";
+                }
+                lineLength = 0;
+                // Let's check if the next word is not too long for the new line to contain
+                // If it is, cut it
+                while (word.size() > lnSize)
+                {
+                    QString wordPart = word;
+                    wordPart.resize(lnSize);
+                    word.remove(0,lnSize);
+                    textInLines += wordPart + "<br/>";
+                }
+            }
+            textInLines += word + " ";
+            lineLength += word.size() + 1;
+            words.removeFirst();
+        }
+        return textInLines.trimmed();
+    }
+}
+
+/**
+* Prepares a row for the playlist tooltips consisting of an icon representing
+* an mp3 tag and its value
+* @param column The colunm used to display the icon
+* @param value The QString value to be shown
+* @return The line to be shown or an empty QString if the value is null
+*/
+static QString
+HTMLLine( const Playlist::Column& column, const QString& value, bool force = false )
+{
+    if( !value.isEmpty() || force )
+    {
+        QString line;
+        line += "<tr><td align=\"right\">";
+        line += "<img src=\""+KIconLoader::global()->iconPath( Playlist::iconNames[column] , -16)+"\" />";
+        line += "</td><td align=\"left\">";
+        line += breakLongLinesHTML( value );
+        line += "</td></tr>";
+        return line;
+    }
+    else
+        return QString();
+}
+
+/**
+* Prepares a row for the playlist tooltips consisting of an icon representing
+* an mp3 tag and its value
+* @param column The colunm used to display the icon
+* @param value The integer value to be shown
+* @return The line to be shown or an empty QString if the value is 0
+*/
+static QString
+HTMLLine( const Playlist::Column& column, const int value, bool force = false )
+{
+    if( (value != 0) || force )
+    {
+        return HTMLLine( column, QString::number( value ) );
+    }
+    else
+        return QString();
+}
+
 
 Playlist::Model::Model( QObject *parent )
         : QAbstractListModel( parent )
@@ -67,7 +163,7 @@ Playlist::Model::~Model()
     DEBUG_BLOCK
 
     // Save current playlist
-    exportPlaylist( defaultPlaylistPath() );
+    exportPlaylist( Amarok::defaultPlaylistPath() );
 
     qDeleteAll( m_items );
 }
@@ -80,7 +176,69 @@ Playlist::Model::headerData( int section, Qt::Orientation orientation, int role
     if ( role != Qt::DisplayRole )
         return QVariant();
 
-    return columnNames[section];
+    return columnNames( section );
+}
+
+void
+Playlist::Model::setTooltipColumns( bool columns[] )
+{
+    for( int i=0; i<Playlist::NUM_COLUMNS; ++i )
+        s_tooltipColumns[i] = columns[i];
+}
+
+QString
+Playlist::Model::tooltipFor( Meta::TrackPtr track ) const
+{
+    QString text;
+    // get the shared pointers now to be thread safe
+    Meta::ArtistPtr artist = track->artist();
+    Meta::AlbumPtr album = track->album();
+    Meta::GenrePtr genre = track->genre();
+    Meta::ComposerPtr composer = track->composer();
+    Meta::YearPtr year = track->year();
+
+    if( s_tooltipColumns[Playlist::Title] )
+        text += HTMLLine( Playlist::Title, track->name() );
+
+    if( s_tooltipColumns[Playlist::Artist] && artist )
+        text += HTMLLine( Playlist::Artist, artist->name() );
+
+    if( s_tooltipColumns[Playlist::Album] && album )
+        text += HTMLLine( Playlist::Album, album->name() );
+
+    if( s_tooltipColumns[Playlist::DiscNumber] )
+        text += HTMLLine( Playlist::DiscNumber, track->discNumber() );
+
+    if( s_tooltipColumns[Playlist::TrackNumber] )
+        text += HTMLLine( Playlist::TrackNumber, track->trackNumber() );
+
+    if( s_tooltipColumns[Playlist::Composer] && composer )
+        text += HTMLLine( Playlist::Composer, composer->name() );
+
+    if( s_tooltipColumns[Playlist::Genre] && genre )
+        text += HTMLLine( Playlist::Genre, genre->name() );
+
+    if( s_tooltipColumns[Playlist::Year] && year && year->year() > 0 )
+        text += HTMLLine( Playlist::Year, year->year() );
+
+    if( s_tooltipColumns[Playlist::Comment])
+        text += HTMLLine( Playlist::Comment, track->comment() );
+
+    if( s_tooltipColumns[Playlist::Score] )
+        text += HTMLLine( Playlist::Score, track->score() );
+
+    if( s_tooltipColumns[Playlist::Rating] )
+        text += HTMLLine( Playlist::Rating, QString::number( static_cast<double>(track->rating())/2.0 ) );
+
+    if( s_tooltipColumns[Playlist::PlayCount] )
+        text += HTMLLine( Playlist::PlayCount, track->playCount(), true );
+
+    if( text.isEmpty() )
+        text = QString( i18n( "No extra information available" ) );
+    else
+        text = QString("<table>"+ text +"</table>");
+
+    return text;
 }
 
 QVariant
@@ -115,23 +273,26 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
     else if ( role == StopAfterTrackRole )
         return Actions::instance()->willStopAfterTrack( idAt( row ) );
 
-    else if ( role == Qt::DisplayRole || role == Qt::ToolTipRole )
+    else if ( role == Qt::ToolTipRole )
+        return tooltipFor( m_items.at( row )->track() );
+
+    else if ( role == Qt::DisplayRole )
     {
+        Meta::AlbumPtr album = m_items.at( row )->track()->album();
         switch ( index.column() )
         {
             case PlaceHolder:
-                return QString();
+                return "Test";
             case Album:
             {
-                if ( m_items.at( row )->track()->album() )
-                    return m_items.at( row )->track()->album()->name();
+                if( album )
+                    return album->name();
                 return QString();
             }
             case AlbumArtist:
             {
-                if ( m_items.at( row )->track()->album() )
-                    if (  m_items.at( row )->track()->album()->albumArtist() )
-                        return m_items.at( row )->track()->album()->albumArtist()->name();
+                if( album && album->albumArtist() )
+                    return album->albumArtist()->name();
                 return QString();
             }
             case Artist:
@@ -162,8 +323,8 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
             }
             case CoverImage:
             {
-                if ( m_items.at( row )->track()->album() )
-                    return The::svgHandler()->imageWithBorder( m_items.at( row )->track()->album(), 100 ); //FIXME:size?
+                if( album )
+                    return The::svgHandler()->imageWithBorder( album, 100 ); //FIXME:size?
                 return QImage();
             }
             case Directory:
@@ -219,12 +380,10 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
             }
             case LastPlayed:
             {
-                if ( m_items.at( row )->track()->lastPlayed() == 0 )
-                    return i18n( "Never" );
-                else {
-                    QDateTime date = QDateTime::fromTime_t( m_items.at( row )->track()->lastPlayed() );
-                    return Amarok::verboseTimeSince( date );
-                }
+                if( m_items.at( row )->track()->lastPlayed().isValid() )
+                    return Amarok::verboseTimeSince( m_items.at( row )->track()->lastPlayed() );
+                else
+                    return i18nc( "The amount of time since last played", "Never" );
             }
             case Length:
             {
@@ -452,33 +611,15 @@ Playlist::Model::setActiveRow( int row )
 }
 
 void
-Playlist::Model::setRowQueued( int row )
+Playlist::Model::emitQueueChanged()
 {
-    if( rowExists( row ) )
-    {
-        Item::State state = stateOfRow(row);
-        if( state == Item::Invalid )
-            state = Item::Queued;
-        else
-            state = (Item::State) ( state | Item::Queued );
-        setStateOfRow( row, state );
-        emit queueChanged();
-    }
+    emit queueChanged();
 }
 
-void
-Playlist::Model::setRowDequeued( int row )
+int
+Playlist::Model::queuePositionOfRow( int row )
 {
-    if( rowExists( row ) )
-    {
-        Item::State state = stateOfRow(row);
-        if( state == Item::Queued )
-            state = Item::Invalid;
-        else
-            state = (Item::State) ( stateOfRow(row) & ~Item::Queued );
-        setStateOfRow( row, state );
-        emit queueChanged();
-    }
+    return Actions::instance()->queuePosition( idAt( row ) ) + 1;
 }
 
 Playlist::Item::State
@@ -594,29 +735,47 @@ Playlist::Model::stateOfId( quint64 id ) const
 void
 Playlist::Model::metadataChanged( Meta::TrackPtr track )
 {
-    DEBUG_BLOCK
+    DEBUG_BLOCK;
 
-    const int size = m_items.size();
-    for ( int i = 0; i < size; i++ )
+    int row = 0;
+    foreach( Item* i, m_items )
     {
-        if ( m_items.at( i )->track() == track )
+        if ( i->track() == track )
         {
-            emit dataChanged( index( i, 0 ), index( i, columnCount() - 1 ) );
+            // ensure that we really have the correct album subscribed (in case it changed)
+            Meta::AlbumPtr album = track->album();
+            if( album )
+                subscribeTo( album );
+
+            emit dataChanged( index( row, 0 ), index( row, columnCount() - 1 ) );
             debug()<<"Metadata updated for track"<<track->prettyName();
-            break;
         }
+        row++;
     }
 }
 
 void
 Playlist::Model::metadataChanged( Meta::AlbumPtr album )
 {
-    DEBUG_BLOCK
+    // Mainly to get update about changed covers
 
-    Meta::TrackList tracks = album->tracks();
-    foreach( Meta::TrackPtr track, tracks )
-        metadataChanged( track );
-    debug()<<"Album metadata changed";
+    // -- search for all the tracks having this album
+    bool found = false;
+    const int size = m_items.size();
+    for ( int i = 0; i < size; i++ )
+    {
+        if ( m_items.at( i )->track()->album() == album )
+        {
+            emit dataChanged( index( i, 0 ), index( i, columnCount() - 1 ) );
+            found = true;
+            debug()<<"Metadata updated for album"<<album->prettyName();
+        }
+    }
+
+    // -- unsubscribe if we don't have a track from that album left.
+    // this can happen if the album of a track changed
+    if( !found )
+        unsubscribeFrom( album );
 }
 
 bool
@@ -640,6 +799,74 @@ Playlist::Model::tracks() const
     return tl;
 }
 
+QString
+Playlist::Model::generatePlaylistName() const
+{
+    QString datePart = KGlobal::locale()->formatDateTime( QDateTime::currentDateTime(),
+                                                          KLocale::ShortDate, true );
+    Meta::TrackList trackList = tracks();
+
+    if( trackList.count() == 0 )
+    {
+        return i18nc( "A saved playlist with the current time (KLocale::Shortdate) added between \
+                      the parentheses",
+                      "Empty Playlist (%1)").arg( datePart );
+    }
+
+    bool singleArtist = true;
+    bool singleAlbum = true;
+
+    Meta::ArtistPtr artist = trackList.first()->artist();
+    Meta::AlbumPtr album = trackList.first()->album();
+
+    QString artistPart;
+    QString albumPart;
+
+    foreach( const Meta::TrackPtr track, trackList )
+    {
+        if( artist != track->artist() )
+            singleArtist = false;
+
+        if( album != track->album() )
+            singleAlbum = false;
+
+        if ( !singleArtist && !singleAlbum )
+            break;
+    }
+
+    if( ( !singleArtist && !singleAlbum ) ||
+        ( !artist && !album ) )
+        return i18nc( "A saved playlist with the current time (KLocale::Shortdate) added between \
+                      the parentheses",
+                      "Various Tracks (%1)" ).arg( datePart );
+
+    if( singleArtist )
+    {
+        if( artist )
+            artistPart = artist->prettyName();
+        else
+            artistPart = i18n( "Unknown Artist(s)" );
+    }
+    else if( album && album->hasAlbumArtist() && singleAlbum )
+        artistPart = album->albumArtist()->prettyName();
+    else
+        artistPart = i18n( "Various Artists" );
+
+    if( singleAlbum )
+    {
+        if( album )
+            albumPart = album->prettyName();
+        else
+            albumPart = i18n( "Unknown Album(s)" );
+    }
+    else
+        albumPart = i18n( "Various Albums" );
+
+    return i18nc( "A saved playlist titled <artist> - <album>", "%1 - %2")
+            .arg( artistPart, albumPart );
+
+}
+
 QString
 Playlist::Model::prettyColumnName( Column index ) //static
 {
@@ -705,9 +932,9 @@ Playlist::Model::insertTracksCommand( const InsertCmdList& cmds )
         m_totalLength += track->length();
         m_totalSize += track->filesize();
         subscribeTo( track );
-
-        if ( track->album() )
-            subscribeTo( track->album() );
+        Meta::AlbumPtr album = track->album();
+        if( album )
+            subscribeTo( album );
 
         Item* newitem = new Item( track );
         m_items.insert( ic.second, newitem );
@@ -780,6 +1007,7 @@ Playlist::Model::removeTracksCommand( const RemoveCmdList& cmds )
         Item* item = originalList.at(rc.second);
         Q_ASSERT( track == item->track() );
 
+        // -- remove the items from the lists
         int idx = rowForItem( item );
         if (idx != -1) {
             beginRemoveRows(QModelIndex(), idx, idx);
@@ -794,11 +1022,28 @@ Playlist::Model::removeTracksCommand( const RemoveCmdList& cmds )
         m_totalLength -= track->length();
         m_totalSize -= track->filesize();
 
+        // -- unsubscribe
         if( !containsTrack( track ) ) // check against same track two times in playlist
         {
             unsubscribeFrom( track );
-            if ( track->album() )
-                unsubscribeFrom( track->album() );
+
+            Meta::AlbumPtr album = track->album();
+            if( album )
+            {
+                // check if we are the last track in playlist with this album
+                bool last = true;
+                const int size = m_items.size();
+                for ( int i = 0; i < size; i++ )
+                {
+                    if( m_items.at( i )->track()->album() == album )
+                    {
+                        last = false;
+                        break;
+                    }
+                }
+                if( last )
+                    unsubscribeFrom( album );
+            }
         }
     }
 
