@@ -22,6 +22,7 @@
 #include "Matching.h"
 
 #include "core/meta/Meta.h"
+#include "widgets/MetaQueryWidget.h"
 
 #include <QAbstractListModel>
 #include <QBitArray>
@@ -46,12 +47,11 @@ namespace ConstraintTypes {
      * "last played" that aren't in the actual file that corresponds to the
      * track being played. -- sth */
 
-    class TagMatch : public MatchingConstraint {
+    class TagMatch : public MatchingConstraint
+    {
         Q_OBJECT
 
         public:
-            enum FieldTypes { FieldTypeInt, FieldTypeDate, FieldTypeString };
-
             static Constraint* createFromXml(QDomElement&, ConstraintNode*);
             static Constraint* createNew(ConstraintNode*);
             static ConstraintFactoryEntry* registerMe();
@@ -74,16 +74,18 @@ namespace ConstraintTypes {
 
             virtual ConstraintNode::Vote* vote( const Meta::TrackList&, const Meta::TrackList& ) const;
 
+#ifndef KDE_NO_DEBUG_OUTPUT
+            virtual void audit( const Meta::TrackList& ) const;
+#endif
+
             // Implementation of MatchingConstraint virtuals
             const QBitArray whatTracksMatch( const Meta::TrackList& );
             int constraintMatchType() const;
 
         private slots:
-            void setComparison( int );
-            void setField( const QString& );
+            void setFilter( const MetaQueryWidget::Filter &filter );
             void setInvert( bool );
             void setStrictness( int );
-            void setValue( const QVariant& );
 
         private:
             TagMatch( QDomElement&, ConstraintNode* );
@@ -91,100 +93,65 @@ namespace ConstraintTypes {
             ~TagMatch();
 
             // constraint parameters
-            int m_comparison;
-            QString m_field;
             bool m_invert;
             double m_strictness;
-            QVariant m_value;
 
-            // convenience class
-            const TagMatchFieldsModel* const m_fieldsModel;
+            MetaQueryWidget::Filter m_filter;
 
             // internal state data
             double m_satisfaction;
 
             // convenience functions
-            double dateComparison( uint ) const;
-            double labelComparison( Meta::TrackPtr ) const;
-            QString comparisonToString() const;
             QString valueToString() const;
 
+            // int conditionToComparison( MetaQueryWidget::FilterCondition condition ) const;
             bool matches( const Meta::TrackPtr ) const; // match values are fuzzily calculated
             mutable QHash<Meta::TrackPtr, bool> m_matchCache; // internal cache for per-track true/false data
+
+            /** Support class that does fuzzy comparisons for the TagMatch. */
+            class Comparer {
+                public:
+                    static int rangeNum( const double strictness, const qint64 field );
+                    static double compareNum( const double test,
+                                              MetaQueryWidget::FilterCondition condition,
+                                              double target,
+                                              const double strictness,
+                                              const qint64 field );
+                    static double compareStr( const QString& test,
+                                              MetaQueryWidget::FilterCondition condition,
+                                              const QString& target );
+
+                    static double compareLabels( const Meta::TrackPtr t,
+                                                 MetaQueryWidget::FilterCondition condition,
+                                                 const QString& target );
+                protected:
+                    /** Returns the normal fuzzy range for a field.
+                        e.g. For valScore (range 0-100) returns 20.0 */
+                    static double fieldWeight( qint64 field );
+                    static double fuzzyProb( const double a, const double b,
+                                             const double strictness, const double weight );
+            };
+
     };
 
-    class TagMatchEditWidget : public QWidget {
+    class TagMatchEditWidget : public QWidget
+    {
         Q_OBJECT
 
         public:
-            TagMatchEditWidget( const int, const QString&, const bool, const int, const QVariant& );
+            TagMatchEditWidget( const MetaQueryWidget::Filter &filter, bool invert, int strictness);
             ~TagMatchEditWidget();
 
         signals:
-            void comparisonChanged( int );
-            void fieldChanged( const QString& );
+            void filterChanged( const MetaQueryWidget::Filter &filter );
             void invertChanged( bool );
             void strictnessChanged( int );
-            void valueChanged( const QVariant& );
-            void updated();
 
         private slots:
-            // comparison
-            void on_comboBox_ComparisonDate_currentIndexChanged( int );
-            void on_comboBox_ComparisonInt_currentIndexChanged( int );
-            void on_comboBox_ComparisonRating_currentIndexChanged( int );
-            void on_comboBox_ComparisonString_currentIndexChanged( int );
-            void on_comboBox_ComparisonTime_currentIndexChanged( int );
-
-            // field
-            void on_comboBox_Field_currentIndexChanged( int );
-
-            // invert
-            void on_checkBox_Invert_clicked( bool );
-
-            // strictness
-            void on_slider_StrictnessDate_valueChanged( int );
-            void on_slider_StrictnessInt_valueChanged( int );
-            void on_slider_StrictnessRating_valueChanged( int );
-            void on_slider_StrictnessTime_valueChanged( int );
-
-            // value
-            void on_kdatewidget_DateSpecific_changed( const QDate& );
-            void on_spinBox_ValueDateValue_valueChanged( int );
-            void on_comboBox_ValueDateUnit_currentIndexChanged( int );
-            void on_spinBox_ValueInt_valueChanged( int );
-            void on_lineEdit_StringValue_textChanged( const QString& );
-            void on_rating_RatingValue_ratingChanged( int );
-            void on_timeEdit_TimeValue_timeChanged( const QTime& );
+            void slotUpdateStrictness();
 
         private:
             Ui::TagMatchEditWidget ui;
-            TagMatchFieldsModel* const m_fieldsModel;
-    };
-
-    class TagMatchFieldsModel : public QAbstractListModel {
-        Q_OBJECT
-
-        public:
-            TagMatchFieldsModel();
-            ~TagMatchFieldsModel();
-
-            // required by QAbstractListModel
-            QVariant data( const QModelIndex&, int role = Qt::DisplayRole ) const;
-            int rowCount( const QModelIndex& parent = QModelIndex() ) const;
-
-            bool contains( const QString& ) const;
-            int index_of( const QString& ) const;
-            QString field_at( int ) const;
-            qint64 meta_value_of( const QString& ) const;
-            QString pretty_name_of( const QString& ) const;
-            TagMatch::FieldTypes type_of( const QString& ) const;
-
-        private:
-            QList<QString> m_fieldNames;
-            QHash<QString, TagMatch::FieldTypes> m_fieldTypes;
-            QHash<QString, qint64> m_fieldMetaValues;
-            QHash<QString, QString> m_fieldPrettyNames;
     };
 
 } // namespace ConstraintTypes
