@@ -20,7 +20,7 @@
 #include "App.h"
 #include "core-impl/collections/support/CollectionManager.h"
 #include "core/collections/support/SqlStorage.h"
-#include "core-impl/collections/sqlcollection/SqlCollectionLocation.h"
+#include "core-impl/collections/db/sql/SqlCollectionLocation.h"
 #include "core/support/Debug.h"
 
 
@@ -124,7 +124,7 @@ namespace AmarokScript
             parentUrl.setPath( dir );
             if ( !AmarokConfig::scanRecursively() )
             {
-                if ( ( dir == path ) || ( dir + '/' == path ) )
+                if ( ( dir == path ) || ( QString( dir + '/' ) == path ) )
                     return true;
             }
             else //scan recursively
