@@ -8,15 +8,16 @@
  *                                                                                      *
  * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
  * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
- * PARTICULAR PURPOSE. See the GNU General Public License for more details.              *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
  *                                                                                      *
  * You should have received a copy of the GNU General Public License along with         *
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
 #include "ServiceCollectionLocation.h"
-#include "Debug.h"
-using namespace Meta;
+#include "core/support/Debug.h"
+
+using namespace Collections;
 
 ServiceCollectionLocation::ServiceCollectionLocation()
     : CollectionLocation()
@@ -56,7 +57,7 @@ ServiceCollectionLocation::getKIOCopyableUrls( const Meta::TrackList &tracks )
     QMap<Meta::TrackPtr, KUrl> urls;
     foreach( Meta::TrackPtr track, tracks )
     {
-        ServiceTrack *servtrack = static_cast<ServiceTrack *>( track.data() );
+        Meta::ServiceTrack *servtrack = static_cast<Meta::ServiceTrack *>( track.data() );
         if( servtrack->isPlayable() )
             urls.insert( track, servtrack->downloadableUrl() );
     }
