@@ -71,22 +71,13 @@ class AMAROK_EXPORT ServiceDisplayInfoProvider
         virtual void processInfoOf( InfoParserBase * infoParser ) = 0;
 };
 
-class AMAROK_EXPORT CustomActionsProvider
+class AMAROK_EXPORT ActionsProvider
 {
     public:
-        CustomActionsProvider() {}
-        virtual ~CustomActionsProvider() {}
+        ActionsProvider() {}
+        virtual ~ActionsProvider() {}
 
-        virtual QList< QAction *> customActions() { DEBUG_BLOCK return QList< QAction *>(); }
-};
-
-class AMAROK_EXPORT CurrentTrackActionsProvider
-{
-    public:
-        CurrentTrackActionsProvider() {}
-        virtual ~CurrentTrackActionsProvider() {}
-
-        virtual QList< QAction *> currentTrackActions() { DEBUG_BLOCK return QList< QAction *>(); }
+        virtual QList< QAction *> actions() { DEBUG_BLOCK return QList< QAction *>(); }
 };
 
 class AMAROK_EXPORT SourceInfoProvider
@@ -107,7 +98,7 @@ class AMAROK_EXPORT BookmarkThisProvider : public QObject
     Q_OBJECT
     public:
 
-        BookmarkThisProvider() : QObject(), m_bookmarkAction( 0 ) {}
+        BookmarkThisProvider() : QObject() {}
         virtual ~BookmarkThisProvider() {}
 
         virtual bool isBookmarkable() const { return false; }
@@ -117,7 +108,7 @@ class AMAROK_EXPORT BookmarkThisProvider : public QObject
         virtual QAction * bookmarkAction() { return 0; };
 
     protected:
-        QAction * m_bookmarkAction;
+        QWeakPointer<QAction> m_bookmarkAction;
 };
 
 
@@ -146,9 +137,8 @@ typedef QList<ServiceYearPtr > ServiceYearList;
 
 class AMAROK_EXPORT ServiceTrack : public Meta::Track,
                                    public ServiceDisplayInfoProvider,
-                                   public CustomActionsProvider,
+                                   public ActionsProvider,
                                    public SourceInfoProvider,
-                                   public CurrentTrackActionsProvider,
                                    public BookmarkThisProvider
 {
     public:
@@ -161,7 +151,6 @@ class AMAROK_EXPORT ServiceTrack : public Meta::Track,
         virtual ~ServiceTrack();
 
         virtual QString name() const;
-        virtual QString prettyName() const;
         virtual KUrl downloadableUrl() const;
         virtual KUrl playableUrl() const;
         virtual QString uidUrl() const;
@@ -183,7 +172,7 @@ class AMAROK_EXPORT ServiceTrack : public Meta::Track,
         virtual void setComposer( const QString &newComposer );
 
         virtual YearPtr year() const;
-        virtual void setYear( const QString &newYear );
+        virtual void setYear( int newYear );
 
         virtual void setTitle( const QString &newTitle );
 
@@ -213,8 +202,8 @@ class AMAROK_EXPORT ServiceTrack : public Meta::Track,
         virtual int discNumber() const;
         virtual void setDiscNumber( int newDiscNumber );
 
-        virtual uint lastPlayed() const;
-        virtual uint firstPlayed() const;
+        virtual QDateTime lastPlayed() const;
+        virtual QDateTime firstPlayed() const;
         virtual int playCount() const;
 
         virtual void finishedPlaying( double playedFraction );
@@ -228,21 +217,18 @@ class AMAROK_EXPORT ServiceTrack : public Meta::Track,
 
         virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const
         {
-            return ( type == Capabilities::Capability::CustomActions ) ||
+            return ( type == Capabilities::Capability::Actions ) ||
                    ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() ) ||
-                   ( type == Capabilities::Capability::CurrentTrackActions ) ||
                    ( type == Capabilities::Capability::BookmarkThis ) ||
                    ( type == Capabilities::Capability::FindInSource && isBookmarkable() );
         }
 
         virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type )
         {
-            if ( type == Capabilities::Capability::CustomActions )
-                return new ServiceCustomActionsCapability( this );
+            if ( type == Capabilities::Capability::Actions )
+                return new ServiceActionsCapability( this );
             else if ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() )
                 return new ServiceSourceInfoCapability( this );
-            else if ( type == Capabilities::Capability::CurrentTrackActions )
-                return new ServiceCurrentTrackActionsCapability( this );
             else if ( type == Capabilities::Capability::BookmarkThis )
                 return new ServiceBookmarkThisCapability( this );
             else if ( type == Capabilities::Capability::FindInSource && isBookmarkable() )
@@ -297,7 +283,7 @@ class AMAROK_EXPORT ServiceTrack : public Meta::Track,
 
 class AMAROK_EXPORT ServiceArtist : public Meta::Artist,
                                     public ServiceDisplayInfoProvider,
-                                    public CustomActionsProvider,
+                                    public ActionsProvider,
                                     public SourceInfoProvider,
                                     public BookmarkThisProvider
 {
@@ -307,7 +293,6 @@ class AMAROK_EXPORT ServiceArtist : public Meta::Artist,
         virtual ~ServiceArtist();
 
         virtual QString name() const;
-        virtual QString prettyName() const;
 
         virtual TrackList tracks();
 
@@ -318,15 +303,15 @@ class AMAROK_EXPORT ServiceArtist : public Meta::Artist,
 
         virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const
         {
-            return ( type == Capabilities::Capability::CustomActions ) ||
+            return ( type == Capabilities::Capability::Actions ) ||
                     ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() ) ||
                     ( type == Capabilities::Capability::BookmarkThis );
         }
 
         virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type )
         {
-            if ( type == Capabilities::Capability::CustomActions )
-                return new ServiceCustomActionsCapability( this );
+            if ( type == Capabilities::Capability::Actions )
+                return new ServiceActionsCapability( this );
             else if ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() )
                 return new ServiceSourceInfoCapability( this );
             else if ( type == Capabilities::Capability::BookmarkThis )
@@ -337,9 +322,9 @@ class AMAROK_EXPORT ServiceArtist : public Meta::Artist,
         virtual QAction * bookmarkAction() {
 
             if ( isBookmarkable() ) {
-                if ( m_bookmarkAction ==  0)
-                    m_bookmarkAction = new BookmarkArtistAction( this, ArtistPtr( this ) );
-                return m_bookmarkAction;
+                if ( m_bookmarkAction.isNull())
+                    m_bookmarkAction = QWeakPointer<QAction>( new BookmarkArtistAction( this, ArtistPtr( this ) ) );
+                return m_bookmarkAction.data();
             }
             else
                 return 0;
@@ -368,7 +353,7 @@ class AMAROK_EXPORT ServiceArtist : public Meta::Artist,
 
 class AMAROK_EXPORT ServiceAlbum : public Meta::Album,
                                    public ServiceDisplayInfoProvider,
-                                   public CustomActionsProvider,
+                                   public ActionsProvider,
                                    public SourceInfoProvider,
                                    public BookmarkThisProvider
 {
@@ -378,7 +363,6 @@ class AMAROK_EXPORT ServiceAlbum : public Meta::Album,
         virtual ~ServiceAlbum();
 
         virtual QString name() const;
-        virtual QString prettyName() const;
 
         virtual bool isCompilation() const;
         virtual bool hasAlbumArtist() const;
@@ -389,15 +373,15 @@ class AMAROK_EXPORT ServiceAlbum : public Meta::Album,
 
         virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const
         {
-            return ( type == Capabilities::Capability::CustomActions ) ||
+            return ( type == Capabilities::Capability::Actions ) ||
                     ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() ) ||
                     ( type == Capabilities::Capability::BookmarkThis );
         }
 
         virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type )
         {
-            if ( type == Capabilities::Capability::CustomActions )
-                return new ServiceCustomActionsCapability( this );
+            if ( type == Capabilities::Capability::Actions )
+                return new ServiceActionsCapability( this );
             else if ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() )
                 return new ServiceSourceInfoCapability( this );
             else if ( type == Capabilities::Capability::BookmarkThis )
@@ -408,9 +392,9 @@ class AMAROK_EXPORT ServiceAlbum : public Meta::Album,
         virtual QAction * bookmarkAction() {
 
             if ( isBookmarkable() ) {
-                if ( m_bookmarkAction ==  0)
-                    m_bookmarkAction = new BookmarkAlbumAction( this, AlbumPtr( this ) );
-                return m_bookmarkAction;
+                if ( m_bookmarkAction.isNull())
+                    m_bookmarkAction = QWeakPointer<QAction>( new BookmarkAlbumAction( this, AlbumPtr( this ) ) );
+                return m_bookmarkAction.data();
             }
             else
                 return 0;
@@ -452,7 +436,7 @@ class AMAROK_EXPORT ServiceAlbum : public Meta::Album,
 
 class AMAROK_EXPORT ServiceGenre : public Meta::Genre,
                                    public ServiceDisplayInfoProvider,
-                                   public CustomActionsProvider,
+                                   public ActionsProvider,
                                    public SourceInfoProvider,
                                    public BookmarkThisProvider
 {
@@ -462,7 +446,6 @@ class AMAROK_EXPORT ServiceGenre : public Meta::Genre,
         virtual ~ServiceGenre();
 
         virtual QString name() const;
-        virtual QString prettyName() const;
 
         virtual TrackList tracks();
 
@@ -470,15 +453,15 @@ class AMAROK_EXPORT ServiceGenre : public Meta::Genre,
 
         virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const
         {
-            return ( type == Capabilities::Capability::CustomActions ) ||
+            return ( type == Capabilities::Capability::Actions ) ||
                     ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() ) ||
                     ( type == Capabilities::Capability::BookmarkThis );
         }
 
         virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type )
         {
-            if ( type == Capabilities::Capability::CustomActions )
-                return new ServiceCustomActionsCapability( this );
+            if ( type == Capabilities::Capability::Actions )
+                return new ServiceActionsCapability( this );
             else if ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() )
                 return new ServiceSourceInfoCapability( this );
             else if ( type == Capabilities::Capability::BookmarkThis )
@@ -509,7 +492,7 @@ class AMAROK_EXPORT ServiceGenre : public Meta::Genre,
 
 class AMAROK_EXPORT ServiceComposer : public Meta::Composer,
                                       public ServiceDisplayInfoProvider,
-                                      public CustomActionsProvider,
+                                      public ActionsProvider,
                                       public SourceInfoProvider,
                                       public BookmarkThisProvider
 {
@@ -518,7 +501,6 @@ class AMAROK_EXPORT ServiceComposer : public Meta::Composer,
         virtual ~ServiceComposer();
 
         virtual QString name() const;
-        virtual QString prettyName() const;
 
         virtual TrackList tracks();
 
@@ -526,15 +508,15 @@ class AMAROK_EXPORT ServiceComposer : public Meta::Composer,
 
         virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const
         {
-            return ( type == Capabilities::Capability::CustomActions ) ||
+            return ( type == Capabilities::Capability::Actions ) ||
                     ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() ) ||
                     ( type == Capabilities::Capability::BookmarkThis );
         }
 
         virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type )
         {
-            if ( type == Capabilities::Capability::CustomActions )
-                return new ServiceCustomActionsCapability( this );
+            if ( type == Capabilities::Capability::Actions )
+                return new ServiceActionsCapability( this );
             else if ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() )
                 return new ServiceSourceInfoCapability( this );
             else if ( type == Capabilities::Capability::BookmarkThis )
@@ -552,7 +534,7 @@ class AMAROK_EXPORT ServiceComposer : public Meta::Composer,
 
 class AMAROK_EXPORT ServiceYear : public Meta::Year,
                                   public ServiceDisplayInfoProvider,
-                                  public CustomActionsProvider,
+                                  public ActionsProvider,
                                   public SourceInfoProvider,
                                   public BookmarkThisProvider
 {
@@ -561,7 +543,6 @@ class AMAROK_EXPORT ServiceYear : public Meta::Year,
         virtual ~ServiceYear();
 
         virtual QString name() const;
-        virtual QString prettyName() const;
 
         virtual TrackList tracks();
 
@@ -569,15 +550,15 @@ class AMAROK_EXPORT ServiceYear : public Meta::Year,
 
         virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const
         {
-            return ( type == Capabilities::Capability::CustomActions ) ||
+            return ( type == Capabilities::Capability::Actions ) ||
                     ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() ) ||
                     ( type == Capabilities::Capability::BookmarkThis );
         }
 
         virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type )
         {
-            if ( type == Capabilities::Capability::CustomActions )
-                return new ServiceCustomActionsCapability( this );
+            if ( type == Capabilities::Capability::Actions )
+                return new ServiceActionsCapability( this );
             else if ( type == Capabilities::Capability::SourceInfo && hasSourceInfo() )
                 return new ServiceSourceInfoCapability( this );
             else if ( type == Capabilities::Capability::BookmarkThis )
