@@ -27,6 +27,7 @@
 #include "ServiceSqlRegistry.h"
 #include "widgets/SearchWidget.h"
 
+#include <KAction>
 #include <KMenuBar>
 #include <KRun>
 #include <KShell>
@@ -34,10 +35,9 @@
 #include <KTemporaryFile>
 #include <threadweaver/ThreadWeaver.h>
 
-#include <QAction>
+#include <QToolButton>
 
 #include <typeinfo>
-#include <KAction>
 
 using namespace Meta;
 
