@@ -32,7 +32,6 @@
 
 ProgressWidget::ProgressWidget( QWidget *parent )
         : QWidget( parent )
-        , Engine::EngineObserver( The::engineController() )
 {
 
     QHBoxLayout *box = new QHBoxLayout( this );
@@ -63,10 +62,31 @@ ProgressWidget::ProgressWidget( QWidget *parent )
     box->addWidget( m_slider );
     box->addWidget( m_timeLabelRight );
 
-    engineStateChanged( Phonon::StoppedState );
+    EngineController *engine = The::engineController();
 
-    connect( m_slider, SIGNAL( sliderReleased( int ) ), The::engineController(), SLOT( seek( int ) ) );
-    connect( m_slider, SIGNAL( valueChanged( int ) ), SLOT( drawTimeDisplay( int ) ) );
+    if( engine->isPaused() )
+        paused();
+    else if( engine->isPlaying() )
+        trackPlaying();
+    else
+        stopped();
+
+    connect( engine, SIGNAL( stopped( qint64, qint64 ) ),
+             this, SLOT( stopped() ) );
+    connect( engine, SIGNAL( paused() ),
+             this, SLOT( paused() ) );
+    connect( engine, SIGNAL( trackPlaying( Meta::TrackPtr ) ),
+             this, SLOT( trackPlaying() ) );
+    connect( engine, SIGNAL( trackLengthChanged( qint64 ) ),
+             this, SLOT( trackLengthChanged( qint64 ) ) );
+    connect( engine, SIGNAL( trackPositionChanged( qint64, bool ) ),
+             this, SLOT( trackPositionChanged( qint64 ) ) );
+
+    connect( m_slider, SIGNAL( sliderReleased( int ) ),
+             engine, SLOT( seek( int ) ) );
+
+    connect( m_slider, SIGNAL( valueChanged( int ) ),
+             SLOT( drawTimeDisplay( int ) ) );
 
     setBackgroundRole( QPalette::BrightText );
 
@@ -95,156 +115,88 @@ ProgressWidget::drawTimeDisplay( int ms )  //SLOT
 {
     if ( !isVisible() )
         return;
-    
-    int seconds = ms / 1000;
-    int seconds2 = seconds; // for the second label
 
     const qint64 trackLength = The::engineController()->trackLength();
 
-    QString s1;
-    QString s2;
-
     //sometimes the engine gives negative position and track length values for streams
     //which causes the time sliders to show 'interesting' values like -322:0-35:0-59
-    if( ( ms > 1 ) && ( trackLength > 1 ) )
+    int seconds = qMax(0, ms / 1000);
+    int remainingSeconds = qMax(0, int((trackLength - ms) / 1000));
+
+    QString sSeconds = Meta::secToPrettyTime( seconds );
+    QString sRemainingSeconds = "-" + Meta::secToPrettyTime( remainingSeconds );
+
+    if( AmarokConfig::leftTimeDisplayRemaining() )
     {
+        m_timeLabelLeft->setText( sRemainingSeconds );
+        m_timeLabelLeft->setEnabled( remainingSeconds > 0 );
 
-        // when the left label shows the remaining time and it's not a stream
-        if ( AmarokConfig::leftTimeDisplayRemaining() && trackLength > 0 )
-        {
-            seconds2 = seconds;
-            seconds = ( trackLength / 1000 ) - seconds;
-        }
-
-        // when the left label shows the remaining time and it's a stream
-        else if ( AmarokConfig::leftTimeDisplayRemaining() && trackLength == 0 )
-        {
-            seconds2 = seconds;
-            seconds = 0; // for streams
-        }
-
-        // when the right label shows the remaining time and it's not a stream
-        else if ( !AmarokConfig::leftTimeDisplayRemaining() && trackLength > 0 )
-        {
-            seconds2 = ( trackLength / 1000 ) - seconds;
-        }
-
-        // when the right label shows the remaining time and it's a stream
-        else if ( !AmarokConfig::leftTimeDisplayRemaining() && trackLength == 0 )
-        {
-            seconds2 = 0;
-        }
-
-        //put Utility functions somewhere
-        s1 = Meta::secToPrettyTime( seconds );
-        s2 = Meta::secToPrettyTime( seconds2 );
-
-        // when the left label shows the remaining time and it's not a stream
-        if ( AmarokConfig::leftTimeDisplayRemaining() && trackLength > 0 )
-            s1.prepend( '-' );
-
-        // when the right label shows the remaining time and it's not a stream
-        else if ( !AmarokConfig::leftTimeDisplayRemaining() && trackLength > 0 )
-            s2.prepend( '-' );
-
-        m_timeLabelLeft->setText( s1 );
-        m_timeLabelRight->setText( s2 );
-
-
-        if ( AmarokConfig::leftTimeDisplayRemaining() && trackLength < 1 )
-        {
-            m_timeLabelLeft->setEnabled( false );
-            m_timeLabelRight->setEnabled( true );
-        }
-        else if ( !AmarokConfig::leftTimeDisplayRemaining() && trackLength < 1 )
-        {
-            m_timeLabelLeft->setEnabled( true );
-            m_timeLabelRight->setEnabled( false );
-        }
-        else
-        {
-            m_timeLabelLeft->setEnabled( true );
-            m_timeLabelRight->setEnabled( true );
-        }
-    
+        m_timeLabelRight->setText( sSeconds );
+        m_timeLabelRight->setEnabled( seconds > 0 );
     }
     else
     {
-       m_timeLabelLeft->setEnabled( false );
-       m_timeLabelRight->setEnabled( false );
-    }
-    
+        m_timeLabelRight->setText( sRemainingSeconds );
+        m_timeLabelRight->setEnabled( remainingSeconds > 0 );
 
-
-}
-
-void
-ProgressWidget::engineTrackPositionChanged( qint64 position, bool /*userSeek*/ )
-{
-    //debug() << "POSITION: " << position;
-    m_slider->setSliderValue( position );
-
-    if ( !m_slider->isEnabled() )
-        drawTimeDisplay( position );
-}
-
-void
-ProgressWidget::engineStateChanged( Phonon::State state, Phonon::State /*oldState*/ )
-{
-    switch ( state )
-    {
-        case Phonon::LoadingState:
-            if ( !The::engineController()->currentTrack() || ( m_currentUrlId != The::engineController()->currentTrack()->uidUrl() ) )
-            {
-                m_slider->setEnabled( false );
-                debug() << "slider disabled!";
-                m_slider->setMinimum( 0 ); //needed because setMaximum() calls with bogus values can change minValue
-                m_slider->setMaximum( 0 );
-                m_timeLabelLeft->setEnabled( false );
-                m_timeLabelLeft->setEnabled( false );
-                m_timeLabelLeft->setShowTime( false );
-                m_timeLabelRight->setShowTime( false );
-            }
-            break;
-
-        case Phonon::PlayingState:
-
-            //in some cases (for streams mostly), we do not get an event for track length changes once
-            //loading is done, causing maximum() to return 0 at when playback starts. In this case we need
-            //to make sure that maximum is set correctly or the slider will not move.
-            if( m_slider->maximum() == 0 )
-                m_slider->setMaximum( The::engineController()->trackLength() );
-
-            m_timeLabelLeft->setEnabled( true );
-            m_timeLabelLeft->setEnabled( true );
-            m_timeLabelLeft->setShowTime( true );
-            m_timeLabelRight->setShowTime( true );
-            //fallthrough
-            break;
-
-        case Phonon::StoppedState:
-        case Phonon::BufferingState:
-        case Phonon::ErrorState:
-            break;
-
-        case Phonon::PausedState:
-            m_timeLabelLeft->setEnabled( true );
-            m_timeLabelRight->setEnabled( true );
-            break;
+        m_timeLabelLeft->setText( sSeconds );
+        m_timeLabelLeft->setEnabled( seconds > 0 );
     }
 }
 
 void
-ProgressWidget::engineTrackLengthChanged( qint64 milliseconds )
+ProgressWidget::stopped()
+{
+    m_slider->setEnabled( false );
+    m_slider->setMinimum( 0 ); //needed because setMaximum() calls with bogus values can change minValue
+    m_slider->setMaximum( 0 );
+    m_timeLabelLeft->setEnabled( false );
+    m_timeLabelLeft->setEnabled( false );
+    m_timeLabelLeft->setShowTime( false );
+    m_timeLabelRight->setShowTime( false );
+
+    m_currentUrlId.clear();
+    m_slider->clearTriangles();
+}
+
+void
+ProgressWidget::paused()
+{
+    // I am wondering, is there a way that the track can get paused
+    // directly?
+    m_timeLabelLeft->setEnabled( true );
+    m_timeLabelRight->setEnabled( true );
+}
+
+void
+ProgressWidget::trackPlaying()
+{
+    //in some cases (for streams mostly), we do not get an event for track length changes once
+    //loading is done, causing maximum() to return 0 at when playback starts. In this case we need
+    //to make sure that maximum is set correctly or the slider will not move.
+    m_slider->setEnabled( false );
+    if( m_slider->maximum() == 0 )
+        m_slider->setMaximum( The::engineController()->trackLength() );
+
+    m_timeLabelLeft->setEnabled( true );
+    m_timeLabelLeft->setEnabled( true );
+    m_timeLabelLeft->setShowTime( true );
+    m_timeLabelRight->setShowTime( true );
+
+    trackLengthChanged( The::engineController()->trackLength() );
+}
+
+void
+ProgressWidget::trackLengthChanged( qint64 milliseconds )
 {
     DEBUG_BLOCK
 
     debug() << "new length: " << milliseconds;
     m_slider->setMinimum( 0 );
     m_slider->setMaximum( milliseconds );
-    m_slider->setEnabled( milliseconds > 0 );
+    m_slider->setEnabled( (milliseconds > 0) && The::engineController()->isSeekable() );
     debug() << "slider enabled!";
-    
+
     const int timeLength = Meta::msToPrettyTime( milliseconds ).length() + 1; // account for - in remaining time
     QFontMetrics tFm( m_timeLabelRight->font() );
     const int labelSize = tFm.width(QChar('0')) * timeLength;
@@ -262,6 +214,17 @@ ProgressWidget::engineTrackLengthChanged( qint64 milliseconds )
     redrawBookmarks();
 }
 
+void
+ProgressWidget::trackPositionChanged( qint64 position )
+{
+    //debug() << "POSITION: " << position;
+    m_slider->setSliderValue( position );
+
+    if ( !m_slider->isEnabled() )
+        drawTimeDisplay( position );
+}
+
+
 void
 ProgressWidget::redrawBookmarks( const QString *BookmarkName )
 {
@@ -293,37 +256,10 @@ ProgressWidget::redrawBookmarks( const QString *BookmarkName )
     }
 }
 
-void
-ProgressWidget::engineNewTrackPlaying()
-{
-    DEBUG_BLOCK
-    m_slider->setEnabled( false );
-    engineTrackLengthChanged( The::engineController()->trackLength() );
-}
-
 QSize ProgressWidget::sizeHint() const
 {
     //int height = fontMetrics().boundingRect( "123456789:-" ).height();
     return QSize( width(), 12 );
 }
 
-void ProgressWidget::enginePlaybackEnded( qint64 finalPosition, qint64 trackLength, PlaybackEndedReason reason )
-{
-    Q_UNUSED( finalPosition )
-    Q_UNUSED( trackLength )
-    Q_UNUSED( reason )
-    DEBUG_BLOCK
-
-    m_slider->setEnabled( false );
-    m_slider->setMinimum( 0 ); //needed because setMaximum() calls with bogus values can change minValue
-    m_slider->setMaximum( 0 );
-    m_timeLabelLeft->setEnabled( false );
-    m_timeLabelLeft->setEnabled( false );
-    m_timeLabelLeft->setShowTime( false );
-    m_timeLabelRight->setShowTime( false );
-
-    m_currentUrlId.clear();
-    m_slider->clearTriangles();
-}
-
 #include "ProgressWidget.moc"
